@@ -12,6 +12,7 @@ extern "C" {
 #include "AlignmentGroup.hpp"
 #include "AlignmentModel.hpp"
 #include "BAMQueue.hpp"
+#include "BAMUtils.hpp"
 #include "ClusterForest.hpp"
 #include "DistributionUtils.hpp"
 #include "EquivalenceClassBuilder.hpp"
@@ -56,7 +57,7 @@ public:
       : alignmentFiles_(alnFiles), transcriptFile_(transcriptFile),
         libFmt_(libFmt), transcripts_(std::vector<Transcript>()),
         fragStartDists_(5), posBiasFW_(5), posBiasRC_(5), posBiasExpectFW_(5),
-        posBiasExpectRC_(5), seqBiasModel_(1.0),
+        posBiasExpectRC_(5), /*seqBiasModel_(1.0),*/
         eqBuilder_(salmonOpts.jointLog, salmonOpts.maxHashResizeThreads), quantificationPasses_(0),
         expectedBias_(constExprPow(4, readBias_[0].getK()), 1.0),
         expectedGC_(salmonOpts.numConditionalGCBins, salmonOpts.numFragGCBins,
@@ -105,6 +106,9 @@ public:
 
     SAM_hdr* header = bq->header();
 
+    // Figure out aligner information from the header if we can
+    aligner_ = salmon::bam_utils::inferAlignerFromHeader(header);
+
     // The transcript file existed, so load up the transcripts
     double alpha = 0.005;
     for (decltype(header->nref) i = 0; i < header->nref; ++i) {
@@ -154,8 +158,8 @@ for (auto& txp : transcripts_) {
 
     // Initialize the fragment length distribution
     size_t maxFragLen = salmonOpts.fragLenDistMax;
-    size_t meanFragLen = salmonOpts.fragLenDistPriorMean;
-    size_t fragLenStd = salmonOpts.fragLenDistPriorSD;
+    double meanFragLen = salmonOpts.fragLenDistPriorMean;
+    double fragLenStd = salmonOpts.fragLenDistPriorSD;
     size_t fragLenKernelN = 4;
     double fragLenKernelP = 0.5;
     flDist_.reset(new FragmentLengthDistribution(1.0, maxFragLen, meanFragLen,
@@ -194,18 +198,77 @@ for (auto& txp : transcripts_) {
           pmf, DistributionSpace::LINEAR);
     }
 
+    salmon::utils::markAuxiliaryTargets(salmonOpts, transcripts_);
+
     // Start parsing the alignments
     NullFragmentFilter<FragT>* nff = nullptr;
     bool onlyProcessAmbiguousAlignments = false;
     bq->start(nff, onlyProcessAmbiguousAlignments);
   }
 
+  AlignmentLibrary(std::vector<boost::filesystem::path>& alnFiles,
+                   LibraryFormat libFmt, SalmonOpts& salmonOpts,
+                   bool /*eqClassMode_*/, std::vector<std::string>& tnames,
+                   std::vector<double>& tefflens)
+      : alignmentFiles_(alnFiles),
+        libFmt_(libFmt), transcripts_(std::vector<Transcript>()),
+        fragStartDists_(5), posBiasFW_(5), posBiasRC_(5), posBiasExpectFW_(5),
+        posBiasExpectRC_(5), /*seqBiasModel_(1.0),*/
+        eqBuilder_(salmonOpts.jointLog, salmonOpts.maxHashResizeThreads), quantificationPasses_(0),
+        expectedBias_(constExprPow(4, readBias_[0].getK()), 1.0),
+        expectedGC_(salmonOpts.numConditionalGCBins, salmonOpts.numFragGCBins,
+                    distribution_utils::DistributionSpace::LOG),
+        observedGC_(salmonOpts.numConditionalGCBins, salmonOpts.numFragGCBins,
+                    distribution_utils::DistributionSpace::LOG) {
+    namespace bfs = boost::filesystem;
+
+    // Make sure the alignment file exists.
+    for (auto& alignmentFile : alignmentFiles_) {
+      if (!bfs::exists(alignmentFile)) {
+        std::stringstream ss;
+        ss << "The provided eqClass file: " << alignmentFile
+           << " does not exist!\n";
+        throw std::invalid_argument(ss.str());
+      }
+    }
+
+    // The transcript file existed, so load up the transcripts
+    double alpha = 0.005;
+    for (size_t i = 0; i < tnames.size(); ++i) {
+      transcripts_.emplace_back(i, tnames[i].c_str(), tefflens[i], true, alpha);
+    }
+
+    // Initialize the fragment length distribution
+    size_t maxFragLen = salmonOpts.fragLenDistMax;
+    double meanFragLen = salmonOpts.fragLenDistPriorMean;
+    double fragLenStd = salmonOpts.fragLenDistPriorSD;
+    size_t fragLenKernelN = 4;
+    double fragLenKernelP = 0.5;
+    flDist_.reset(new FragmentLengthDistribution(1.0, maxFragLen, meanFragLen,
+                                                 fragLenStd, fragLenKernelN,
+                                                 fragLenKernelP, 1));
+
+    alnMod_.reset(new AlignmentModel(1.0, salmonOpts.numErrorBins));
+    alnMod_->setLogger(salmonOpts.jointLog);
+    salmon::utils::markAuxiliaryTargets(salmonOpts, transcripts_);
+  }
+
   EQBuilderT& equivalenceClassBuilder() { return eqBuilder_; }
 
   std::string getIndexSeqHash256() const { return ""; }
   std::string getIndexNameHash256() const { return ""; }
   std::string getIndexSeqHash512() const { return ""; }
   std::string getIndexNameHash512() const { return ""; }
+  std::string getIndexDecoySeqHash256() const { return ""; }
+  std::string getIndexDecoyNameHash256() const { return ""; }
+
+  /**
+   * Return true if this read library is for paired-end reads and false
+   * otherwise.
+   */
+  bool isPairedEnd() { return (libFmt_.type == ReadType::PAIRED_END); }
+
+  salmon::bam_utils::AlignerDetails getAlignerType() const { return aligner_; }
 
   // TODO: Make same as mapping-based
   void updateTranscriptLengthsAtomic(std::atomic<bool>& done) {
@@ -283,7 +346,7 @@ for (auto& txp : transcripts_) {
 
   inline AlignmentModel& alignmentModel() { return *alnMod_.get(); }
 
-  SequenceBiasModel& sequenceBiasModel() { return seqBiasModel_; }
+  // SequenceBiasModel& sequenceBiasModel() { return seqBiasModel_; }
 
   //    inline tbb::concurrent_queue<FragT*>& fragmentQueue() {
   inline tbb::concurrent_queue<FragT*>& fragmentQueue() {
@@ -437,7 +500,16 @@ for (auto& txp : transcripts_) {
     return lengthQuantiles_;
   }
 
+  uint64_t getNumDecoys() const {
+    return numDecoys_;
+  }
+
+  salmon::utils::DuplicateTargetStatus index_retains_duplicates() const { 
+    return salmon::utils::DuplicateTargetStatus::UNKNOWN; 
+  }
+
 private:
+
   void setTranscriptLengthClasses_(std::vector<uint32_t>& lengths,
                                    size_t nbins) {
     auto n = lengths.size();
@@ -505,7 +577,7 @@ private:
   // std::unique_ptr<t_pool, std::function<void(t_pool*)>> threadPool_;
   std::unique_ptr<BAMQueue<FragT>> bq;
 
-  SequenceBiasModel seqBiasModel_;
+  // SequenceBiasModel seqBiasModel_;
 
   /**
    * The cluster forest maintains the dynamic relationship
@@ -560,6 +632,9 @@ private:
   std::vector<double> expectedBias_;
   std::unique_ptr<LibraryTypeDetector> detector_{nullptr};
   std::vector<double> conditionalMeans_;
+
+  salmon::bam_utils::AlignerDetails aligner_{salmon::bam_utils::AlignerDetails::UNKNOWN};
+  uint64_t numDecoys_{0};
 };
 
 #endif // ALIGNMENT_LIBRARY_HPP
