@@ -9,9 +9,12 @@ template <typename VecT>
 void EMUpdate_(std::vector<std::vector<uint32_t>>& txpGroupLabels,
                std::vector<std::vector<double>>& txpGroupCombinedWeights,
                const std::vector<uint64_t>& txpGroupCounts,
-               std::vector<Transcript>& transcripts, const VecT& alphaIn,
+               const VecT& alphaIn,
                VecT& alphaOut);
 
+/**
+ * set entries with values <= cutoff to 0.
+ **/
 template <typename VecT>
 double truncateCountVector(VecT& alphas, double cutoff);
 
