@@ -10,6 +10,9 @@
 #include <memory> // for shared_ptr
 #include <ostream>
 
+#include "pufferfish/Util.hpp"
+#include "SalmonDefaults.hpp"
+
 enum class SalmonQuantMode { MAP = 1, ALIGN = 2 };
 
 /**
@@ -39,6 +42,7 @@ struct SalmonOpts {
   SalmonQuantMode quantMode; // How quantification is done
 
   bool alevinMode{false}; // are we running in alevin mode
+  bool eqClassMode{salmon::defaults::eqClassMode}; // are we running in eqclass mode
 
   bool alternativeInitMode; // Weigh unique reads more heavily when initialzing
                             // the optimization.
@@ -57,6 +61,7 @@ struct SalmonOpts {
   bool consistentHits; // Enforce consistency of hits gathered during
                        // quasi-mapping.
 
+  bool skipQuant; // Skip quantification step
   bool dumpEq; // Dump the equivalence classes and counts to file
 
   bool dumpEqWeights; // Dump the equivalence classes rich weights
@@ -73,13 +78,18 @@ struct SalmonOpts {
   double quasiCoverage; // [Experimental]: Default of 0.  The coverage by MMPs
                         // required for a read to be considered mapped.
 
+  std::string hitFilterPolicyStr{salmon::defaults::hitFilterPolicyStr};
+  std::string auxTargetFile{salmon::defaults::auxTargetFile};
+  pufferfish::util::HitFilterPolicy hitFilterPolicy{pufferfish::util::HitFilterPolicy::FILTER_AFTER_CHAINING};
+
   bool splitSpanningSeeds; // Attempt to split seeds that span multiple
                            // transcripts.
 
   bool noFragLengthDist; // Don't give a fragment assignment a likelihood based
                          // on an emperically observed fragment length
                          // distribution.
-
+  bool noSingleFragProb; // Don't attempt to model frag length for single-end
+                         // (or orphaned) mappings.
   bool noEffectiveLengthCorrection; // Don't take the fragment length
                                     // distribution into account when computing
                                     // the probability that a
@@ -98,8 +108,6 @@ struct SalmonOpts {
   bool rankEqClasses; // Keep a separate equivalence class for each ordering (by
                       // conditional probability) of the the transcript labels
 
-  bool useFSPD; // Learn a non-uniform start distribution
-
   bool noSeqBiasModel; // Don't learn and use a sequence-specific bias model.
 
   bool noRichEqClasses; // Don't use rich equivalence classes --- forget the
@@ -153,8 +161,6 @@ struct SalmonOpts {
                           // length pmf when evaluating gc-bias for effective
                           // length correction.
 
-  bool strictIntersect; // Use strict rather than fuzzy intersection in
-                        // quasi-mapping
   bool useMassBanking;  // DEPRECATED
 
   bool sensitive; // Perform splitting of long SMEMs into MEMs
@@ -232,12 +238,13 @@ struct SalmonOpts {
                 // the sequence-specific "foreground" distribution.
 
   // Related to the prior of the VBEM algorithm
-  double vbPrior{1e-3};
-  bool perTranscriptPrior{false};
+  double vbPrior{1e-2};
+  bool perTranscriptPrior{true};
+  bool perNucleotidePrior{false};
   // Related to the fragment length distribution
   size_t fragLenDistMax;
-  size_t fragLenDistPriorMean;
-  size_t fragLenDistPriorSD;
+  double fragLenDistPriorMean;
+  double fragLenDistPriorSD;
 
   // Related to the logger
   std::shared_ptr<spdlog::logger> jointLog{nullptr};
@@ -251,14 +258,29 @@ struct SalmonOpts {
 
   // Related to alignment verification
   bool validateMappings;
-  int32_t consensusSlack;
+  bool disableSA{false}; // this cannot be done right now.
+  float consensusSlack;
+  bool disableChainingHeuristic;
+  bool disableAlignmentCache;
   double minScoreFraction;
+  double scoreExp;
   int16_t matchScore;
   int16_t mismatchPenalty;
   int16_t gapOpenPenalty;
   int16_t gapExtendPenalty;
+  int32_t dpBandwidth;
+  uint32_t mismatchSeedSkip;
   bool mimicStrictBT2;
-  int32_t maxMMPExtension;
+  bool mimicBT2;
+  bool softclip;
+  bool softclipOverhangs;
+  bool fullLengthAlignment;
+  bool allowDovetail;
+  bool recoverOrphans;
+  bool hardFilter;
+  uint32_t maxOccsPerHit;
+  double decoyThreshold;
+  double minAlnProb;
 
   // for utility (may need to be cleaned up later)
   bool discardOrphansQuasi;
