@@ -45,6 +45,17 @@ namespace alevin{
       }
     };
 
+    struct CITESeq : Rule{
+      CITESeq(): Rule(16, 12, BarcodeEnd::FIVE, 4294967295){
+        featureLength = 15;
+        featureStart = 10;
+      }
+
+      size_t featureLength, featureStart;
+      void setFeatureLength(size_t length) { featureLength = length; }
+      void setFeatureStart(size_t startIdx) { featureStart = startIdx; }
+    };
+
     struct ChromiumV3 : Rule{
       ChromiumV3(): Rule(16, 12, BarcodeEnd::FIVE, 4294967295){}
     };
@@ -57,6 +68,10 @@ namespace alevin{
       Gemcode(): Rule(14, 10, BarcodeEnd::FIVE, 268435456){}
     };
 
+    struct QuartzSeq2 : Rule{
+      QuartzSeq2(): Rule(15, 8, BarcodeEnd::FIVE, 1073741824){}
+    };
+
     struct CELSeq : Rule{
       // WEHI SCORE's CEL-Seq2 starts from 5' end with a 8 bp barcode
       // and a 6 bp UMI.
