@@ -15,12 +15,14 @@
 #include <limits>
 #include <memory>
 
-#include "rapmap/bit_array.h"
-#include "rapmap/rank9b.h"
+//#include "rapmap/bit_array.h"
+#include "pufferfish/compact_vector/compact_vector.hpp"
+#include "pufferfish/rank9b.hpp"
 
 class Transcript {
   static constexpr const uint32_t adapterBindingLength{5};
 public:
+  /*
   struct BitArrayDeleter {
     void operator()(BIT_ARRAY* b) {
       if (b != nullptr) {
@@ -28,7 +30,9 @@ public:
       }
     }
   };
-  using BitArrayPointer = std::unique_ptr<BIT_ARRAY, BitArrayDeleter>;
+  */
+  using BitArray = compact::vector<uint64_t,1>;
+  //using BitArrayPointer = std::unique_ptr<BIT_ARRAY, BitArrayDeleter>;
   using Rank9bPointer = std::unique_ptr<rank9b>;
 
   Transcript()
@@ -37,10 +41,9 @@ public:
         EffectiveLength(-1.0), id(std::numeric_limits<uint32_t>::max()),
         logPerBasePrior_(salmon::math::LOG_0), priorMass_(salmon::math::LOG_0),
         mass_(salmon::math::LOG_0), sharedCount_(0.0),
-        avgMassBias_(salmon::math::LOG_0), active_(false) {
+        avgMassBias_(salmon::math::LOG_0), active_(false), skipBiasCorrection_(false) {
     uniqueCount_.store(0);
-    lastUpdate_.store(0);
-    lastTimestepUpdated_.store(0);
+    totalCount_.store(0); // thanks @come-raczy
     cachedEffectiveLength_.store(salmon::math::LOG_0);
   }
 
@@ -48,13 +51,22 @@ public:
       : RefName(name), RefLength(len), CompleteLength(len),
         EffectiveLength(-1.0), id(idIn), logPerBasePrior_(std::log(alpha)),
         priorMass_(std::log(alpha * len)), mass_(salmon::math::LOG_0),
-        sharedCount_(0.0), avgMassBias_(salmon::math::LOG_0), active_(false) {
+        sharedCount_(0.0), avgMassBias_(salmon::math::LOG_0), active_(false), skipBiasCorrection_(false) {
     uniqueCount_.store(0);
-    lastUpdate_.store(0);
-    lastTimestepUpdated_.store(0);
+    totalCount_.store(0); // thanks @come-raczy
     cachedEffectiveLength_.store(std::log(static_cast<double>(RefLength)));
   }
 
+  Transcript(size_t idIn, const char* name, double len, bool /*updateEffLength*/, double alpha = 0.05)
+    : RefName(name), RefLength(len), CompleteLength(len),
+      EffectiveLength(len), id(idIn), logPerBasePrior_(std::log(alpha)),
+      priorMass_(std::log(alpha * len)), mass_(salmon::math::LOG_0),
+      sharedCount_(0.0), avgMassBias_(salmon::math::LOG_0), active_(false), skipBiasCorrection_(false) {
+    uniqueCount_.store(0);
+    totalCount_.store(0); // thanks @come-raczy
+    cachedEffectiveLength_.store(std::log(len));
+  }
+
   // We cannot copy; only move
   Transcript(Transcript& other) = delete;
   Transcript& operator=(Transcript& other) = delete;
@@ -78,10 +90,8 @@ public:
 
     uniqueCount_.store(other.uniqueCount_);
     totalCount_.store(other.totalCount_.load());
-    lastTimestepUpdated_.store(other.lastTimestepUpdated_.load());
     sharedCount_.store(other.sharedCount_.load());
     mass_.store(other.mass_.load());
-    lastUpdate_.store(other.lastUpdate_.load());
     cachedEffectiveLength_.store(other.cachedEffectiveLength_.load());
     lengthClassIndex_ = other.lengthClassIndex_;
     logPerBasePrior_ = other.logPerBasePrior_;
@@ -89,6 +99,8 @@ public:
     avgMassBias_.store(other.avgMassBias_.load());
     hasAnchorFragment_.store(other.hasAnchorFragment_.load());
     active_ = other.active_;
+    isDecoy_ = other.isDecoy_;
+    skipBiasCorrection_ = other.skipBiasCorrection_;
   }
 
   Transcript& operator=(Transcript&& other) {
@@ -109,10 +121,8 @@ public:
 
     uniqueCount_.store(other.uniqueCount_);
     totalCount_.store(other.totalCount_.load());
-    lastTimestepUpdated_.store(other.lastTimestepUpdated_.load());
     sharedCount_.store(other.sharedCount_.load());
     mass_.store(other.mass_.load());
-    lastUpdate_.store(other.lastUpdate_.load());
     cachedEffectiveLength_.store(other.cachedEffectiveLength_.load());
     lengthClassIndex_ = other.lengthClassIndex_;
     logPerBasePrior_ = other.logPerBasePrior_;
@@ -120,6 +130,8 @@ public:
     avgMassBias_.store(other.avgMassBias_.load());
     hasAnchorFragment_.store(other.hasAnchorFragment_.load());
     active_ = other.active_;
+    isDecoy_ = other.isDecoy_;
+    skipBiasCorrection_ = other.skipBiasCorrection_;
     return *this;
   }
 
@@ -178,13 +190,6 @@ public:
     salmon::utils::incLoop(sharedCount_, sc);
   }
 
-  inline void setLastTimestepUpdated(uint64_t currentTimestep) {
-    uint64_t oldTimestep = lastTimestepUpdated_;
-    if (currentTimestep > oldTimestep) {
-      lastTimestepUpdated_ = currentTimestep;
-    }
-  }
-
   inline void addBias(double bias) {
     salmon::utils::incLoopLog(avgMassBias_, bias);
   }
@@ -225,17 +230,13 @@ public:
    *
    */
   double computeLogEffectiveLength(std::vector<double>& logPMF,
-                                   double logFLDMean, size_t minVal,
+                                   size_t minVal,
                                    size_t maxVal) {
 
     double effectiveLength = salmon::math::LOG_0;
     double refLen = static_cast<double>(RefLength);
     double logRefLength = std::log(refLen);
 
-    // JUNE 17 (just ensure it's >= 1)
-    // if (logRefLength <= logFLDMean) {
-    //    effectiveLength = logRefLength;
-    //} else {
     uint32_t mval = maxVal;
     size_t clen = minVal;
     size_t maxLen = std::min(RefLength, mval);
@@ -245,11 +246,9 @@ public:
           effectiveLength, logPMF[i] + std::log(refLen - clen + 1));
       ++clen;
     }
-    //}
     if (salmon::math::isLog0(effectiveLength) or
         std::exp(effectiveLength) < 1.0) {
       effectiveLength = logRefLength;
-      // effectiveLength = //salmon::math::LOG_1;
     }
 
     return effectiveLength;
@@ -264,36 +263,13 @@ public:
     cachedEffectiveLength_.store(l);
   }
 
-  void updateEffectiveLength(std::vector<double>& logPMF, double logFLDMean,
+  void updateEffectiveLength(std::vector<double>& logPMF, double /*logFLDMean*/,
                              size_t minVal, size_t maxVal) {
-    double cel = computeLogEffectiveLength(logPMF, logFLDMean, minVal, maxVal);
+    double cel = computeLogEffectiveLength(logPMF, minVal, maxVal);
     cachedEffectiveLength_.store(cel);
   }
 
-  /**
-   * If we should update the effective length, then do it and cache the result.
-   * Otherwise, return the cached result.
-   */
-  /*
-  double getLogEffectiveLength(const FragmentLengthDistribution& fragLengthDist,
-                               size_t currObs, size_t burnInObs, bool
-  forceUpdate=false) { if (forceUpdate or (lastUpdate_ == 0) or (currObs -
-  lastUpdate_ >= 250000) or (lastUpdate_ < burnInObs and currObs > burnInObs)) {
-          // compute new number
-          lastUpdate_.store(currObs);
-          double cel = computeLogEffectiveLength(fragLengthDist);
-          cachedEffectiveLength_.store(cel);
-          //priorMass_ = cel + logPerBasePrior_;
-          return cachedEffectiveLength_.load();
-      } else {
-          // return cached number
-          return cachedEffectiveLength_.load();
-      }
-  }
-  */
-
   double perBasePrior() { return std::exp(logPerBasePrior_); }
-  inline size_t lastTimestepUpdated() { return lastTimestepUpdated_.load(); }
 
   void lengthClassIndex(uint32_t ind) { lengthClassIndex_ = ind; }
   uint32_t lengthClassIndex() const { return lengthClassIndex_; }
@@ -448,18 +424,26 @@ public:
    * Return the next polyA site that occurs in this transcript
    * after position p
    **/
+  /*
   inline int32_t getNextPolyA(int32_t p) {
     if (p+1 >= static_cast<int32_t>(RefLength)) { return RefLength; }
     auto r = polyARank_->rank(p+1);
     return polyAPos_[r];
   }
+  */
+
+  void setDecoy(bool isDecoy) {
+    isDecoy_ = isDecoy;
+  }
+
+  bool isDecoy() const { return isDecoy_; }
 
   // Will *not* delete seq on destruction
   void setSequenceBorrowed(const char* seq, bool needGC = false,
                            bool reduceGCMemory = false) {
     Sequence_ = std::unique_ptr<const char, void (*)(const char*)>(
         seq,                 // store seq
-        [](const char* p) {} // do nothing deleter
+        [](const char* /*p*/) {} // do nothing deleter
     );
     if (needGC) {
       computeGCContent_(reduceGCMemory);
@@ -506,6 +490,9 @@ public:
 
   void computePolyAPositions() { computePolyAPositions_(); }
 
+  void setSkipBiasCorrection(bool skip) { skipBiasCorrection_ = skip; }
+  bool skipBiasCorrection() const { return skipBiasCorrection_; }
+
   std::string RefName;
   uint32_t RefLength;
   uint32_t CompleteLength;
@@ -538,10 +525,12 @@ private:
 
   inline double gcCountInterp_(int32_t p) const {
     int32_t sRefLength = static_cast<int32_t>(RefLength);
-    if (p >= sRefLength) {
-      p = sRefLength - 1;
+    uint64_t extraCount = 0;
+    if ((p+1) >= sRefLength) {
+      extraCount = gcBitArray_[(RefLength-1)];
+      p = sRefLength - 2;
     }
-    return static_cast<double>(gcRank_->rank(p + 1));
+    return static_cast<double>(gcRank_->rank(p + 1) + extraCount);
   }
 
   /** Previous GC count interp implementation (May 23, 2017) **/
@@ -632,20 +621,23 @@ private:
         GCCount_[i] = totGC;
       }
     } else {
-      BIT_ARRAY* rawArray = bit_array_create(RefLength);
+      gcBitArray_.resize(RefLength);
+      gcBitArray_.clear_mem();
+      //BIT_ARRAY* rawArray = bit_array_create(RefLength);
       for (size_t i = 0; i < RefLength; ++i) {
         auto c = std::toupper(seq[i]);
         if (c == 'G' or c == 'C') {
-          bit_array_set_bit(rawArray, i);
+          gcBitArray_[i] = 1;
+          //bit_array_set_bit(rawArray, i);
         }
       }
-      gcBitArray_.reset(rawArray);
-      gcRank_.reset(new rank9b(gcBitArray_->words, RefLength));
-      // computeGCContentSampled_(gcSampFactor);
+      //gcBitArray_.reset(rawArray);
+      gcRank_.reset(new rank9b(gcBitArray_.get(), RefLength));
     }
   }
 
   void computePolyAPositions_() {
+    /*
     polyAPos_.clear();
     BIT_ARRAY* rawArray = bit_array_create(RefLength);
     std::string polyA(adapterBindingLength, 'A');
@@ -669,15 +661,10 @@ private:
     polyAPos_.push_back(RefLength);
     polyABitArray_.reset(rawArray);
     polyARank_.reset(new rank9b(polyABitArray_->words, RefLength));
-    /*
-    if (polyAPos_.size() >= 3) {
-      std::cerr << polyAView << "; RefLength = " << RefLength << " polyAs = [";
-      for (auto x : polyAPos_) { std::cerr << " " << x; }
-      std::cerr << "]\n";
-    }
     */
   }
 
+
   // NOTE 10.2
   std::vector<uint8_t> SAMSequence_;
 
@@ -687,13 +674,10 @@ private:
 
   std::atomic<size_t> uniqueCount_;
   std::atomic<size_t> totalCount_;
-  // The most recent timestep at which this transcript's mass was updated.
-  std::atomic<size_t> lastTimestepUpdated_;
   double priorMass_;
   tbb::atomic<double> mass_;
   tbb::atomic<double> sharedCount_;
   tbb::atomic<double> cachedEffectiveLength_;
-  tbb::atomic<size_t> lastUpdate_;
   tbb::atomic<double> avgMassBias_;
   uint32_t lengthClassIndex_;
   double logPerBasePrior_;
@@ -702,16 +686,19 @@ private:
   // pair of reads mapping to it.
   std::atomic<bool> hasAnchorFragment_{false};
   bool active_;
+  bool isDecoy_{false};
+  bool skipBiasCorrection_{false};
 
   bool reduceGCMemory_{false};
   double gcFracLen_{0.0};
   uint32_t lastRegularSample_{0};
   std::vector<uint32_t> GCCount_;
-  BitArrayPointer gcBitArray_{nullptr};
+  //BitArrayPointer gcBitArray_{nullptr};
+  BitArray gcBitArray_;
   Rank9bPointer gcRank_{nullptr};
-  BitArrayPointer polyABitArray_{nullptr};
-  Rank9bPointer polyARank_{nullptr};
-  std::vector<int32_t> polyAPos_;
+  //BitArrayPointer polyABitArray_{nullptr};
+  //Rank9bPointer polyARank_{nullptr};
+  //std::vector<int32_t> polyAPos_;
 };
 
 #endif // TRANSCRIPT
