@@ -40,6 +40,68 @@ namespace alevin {
     std::vector<std::string> nts{"A", "T", "C", "G"};
 
 
+    template <>
+    void getReadSequence(apt::CITESeq& protocol,
+                         std::string& seq,
+                         std::string& subseq){
+      subseq = seq.substr(protocol.featureStart,
+                          protocol.featureLength);
+    }
+    template <>
+    void getReadSequence(apt::DropSeq& protocol,
+                         std::string& seq,
+                         std::string& subseq){
+      subseq = seq;
+    }
+    template <>
+    void getReadSequence(apt::Chromium& protocol,
+                         std::string& seq,
+                         std::string& subseq){
+      subseq = seq;
+    }
+    template <>
+    void getReadSequence(apt::ChromiumV3& protocol,
+                         std::string& seq,
+                         std::string& subseq){
+      subseq = seq;
+    }
+    template <>
+    void getReadSequence(apt::CELSeq& protocol,
+                         std::string& seq,
+                         std::string& subseq){
+      subseq = seq;
+    }
+    template <>
+    void getReadSequence(apt::CELSeq2& protocol,
+                         std::string& seq,
+                         std::string& subseq){
+      subseq = seq;
+    }
+    template <>
+    void getReadSequence(apt::QuartzSeq2& protocol,
+                         std::string& seq,
+                         std::string& subseq){
+      subseq = seq;
+    }
+    template <>
+    void getReadSequence(apt::Custom& protocol,
+                         std::string& seq,
+                         std::string& subseq){
+      subseq = seq;
+    }
+    template <>
+    void getReadSequence(apt::Gemcode& protocol,
+                         std::string& seq,
+                         std::string& subseq){
+      subseq = seq;
+    }
+    template <>
+    void getReadSequence(apt::InDrop& protocol,
+                         std::string& seq,
+                         std::string& subseq){
+      subseq = seq;
+    }
+
     template <>
     bool extractUMI<apt::DropSeq>(std::string& read,
                                   apt::DropSeq& pt,
@@ -48,6 +110,13 @@ namespace alevin {
       return true;
     }
     template <>
+    bool extractUMI<apt::CITESeq>(std::string& read,
+                                  apt::CITESeq& pt,
+                                  std::string& umi){
+      umi = read.substr(pt.barcodeLength, pt.umiLength);
+      return true;
+    }
+    template <>
     bool extractUMI<apt::Chromium>(std::string& read,
                                    apt::Chromium& pt,
                                    std::string& umi){
@@ -72,6 +141,19 @@ namespace alevin {
     bool extractUMI<apt::Custom>(std::string& read,
                                  apt::Custom& pt,
                                  std::string& umi){
+      if ( pt.end == BarcodeEnd::FIVE ) {
+        umi = read.substr(pt.barcodeLength, pt.umiLength);
+      } else if (pt.end == BarcodeEnd::THREE ) {
+        umi = read.substr(0, pt.umiLength);
+      } else {
+        return false;
+      }
+      return true;
+    }
+    template <>
+    bool extractUMI<apt::QuartzSeq2>(std::string& read,
+                                     apt::QuartzSeq2& pt,
+                                     std::string& umi){
       umi = read.substr(pt.barcodeLength, pt.umiLength);
       return true;
     }
@@ -94,7 +176,7 @@ namespace alevin {
                                  apt::InDrop& pt,
                                  std::string& umi){
       std::cout<<"Incorrect call for umi extract";
-      exit(0);
+      exit(1);
     }
 
     template <>
@@ -102,44 +184,49 @@ namespace alevin {
                                       apt::DropSeq& pt){
       return (read.length() >= pt.barcodeLength) ?
         nonstd::optional<std::string>(read.substr(0, pt.barcodeLength)) : nonstd::nullopt;
-      // = read.substr(0, pt.barcodeLength);
-    //return true;
+    }
+    template <>
+    nonstd::optional<std::string> extractBarcode<apt::CITESeq>(std::string& read,
+                                                               apt::CITESeq& pt){
+      return (read.length() >= pt.barcodeLength) ?
+        nonstd::optional<std::string>(read.substr(0, pt.barcodeLength)) : nonstd::nullopt;
     }
     template <>
     nonstd::optional<std::string> extractBarcode<apt::ChromiumV3>(std::string& read,
                                                                   apt::ChromiumV3& pt){
       return (read.length() >= pt.barcodeLength) ?
         nonstd::optional<std::string>(read.substr(0, pt.barcodeLength)) : nonstd::nullopt;
-      //return (read.length() >= pt.barcodeLength) ? (bc.append(read.data(), pt.barcodeLength), true) : false;
-      //bc = read.substr(0, pt.barcodeLength);
-      //return true;
     }
     template <>
     nonstd::optional<std::string> extractBarcode<apt::Chromium>(std::string& read,
                                        apt::Chromium& pt){
       return (read.length() >= pt.barcodeLength) ?
         nonstd::optional<std::string>(read.substr(0, pt.barcodeLength)) : nonstd::nullopt;
-      //return (read.length() >= pt.barcodeLength) ? (bc.append(read.data(), pt.barcodeLength), true) : false;
-      //bc = read.substr(0, pt.barcodeLength);
-      //return true;
     }
     template <>
     nonstd::optional<std::string> extractBarcode<apt::Gemcode>(std::string& read,
                                        apt::Gemcode& pt){
       return (read.length() >= pt.barcodeLength) ?
         nonstd::optional<std::string>(read.substr(0, pt.barcodeLength)) : nonstd::nullopt;
-      //return (read.length() >= pt.barcodeLength) ? (bc.append(read.data(), pt.barcodeLength), true) : false;
-      //bc = read.substr(0, pt.barcodeLength);
-      //return true;
     }
     template <>
     nonstd::optional<std::string> extractBarcode<apt::Custom>(std::string& read,
                                      apt::Custom& pt){
+      if (pt.end == BarcodeEnd::FIVE) {
+        return (read.length() >= pt.barcodeLength) ?
+          nonstd::optional<std::string>(read.substr(0, pt.barcodeLength)) : nonstd::nullopt;
+      } else if (pt.end == BarcodeEnd::THREE) {
+        return (read.length() >= (pt.umiLength + pt.barcodeLength)) ?
+          nonstd::optional<std::string>(read.substr(pt.umiLength, pt.barcodeLength)) : nonstd::nullopt;
+      } else {
+        return nonstd::nullopt;
+      }
+    }
+    template <>
+    nonstd::optional<std::string> extractBarcode<apt::QuartzSeq2>(std::string& read,
+                                                                  apt::QuartzSeq2& pt){
       return (read.length() >= pt.barcodeLength) ?
         nonstd::optional<std::string>(read.substr(0, pt.barcodeLength)) : nonstd::nullopt;
-      //return (read.length() >= pt.barcodeLength) ? (bc.append(read.data(), pt.barcodeLength), true) : false;
-      //bc = read.substr(0, pt.barcodeLength);
-      //return true;
     }
     template <>
     nonstd::optional<std::string> extractBarcode<apt::CELSeq2>(std::string& read,
@@ -231,7 +318,7 @@ namespace alevin {
       if(barcodeLength > seqSize){
         std::cout<<"Sequence-Size " << barcodeLength << "greater than specified "
                  << seqSize <<".\nPlease report the issue on Github.\n" ;
-        exit(1);
+        exit(64);
       }
 
       for (size_t i=0; i<barcodeLength; i++){
@@ -250,10 +337,138 @@ namespace alevin {
                         neighbors);
     }
 
+    void getTxpToGeneMap(spp::sparse_hash_map<uint32_t, uint32_t>& txpToGeneMap,
+                         spp::sparse_hash_map<std::string, uint32_t>& geneIdxMap,
+                         const std::string& t2gFileName,
+                         const std::string& refNamesFile,
+                         const std::string& refLengthFile,
+                         const std::string& headerFile,
+                         std::shared_ptr<spdlog::logger>& jointLog,
+                         bool noTgMap){
+      size_t  kSize, numDupTxps{0};
+      uint64_t numberOfDecoys, firstDecoyIndex;
+      spp::sparse_hash_map<std::string, std::vector<uint32_t>> txpIdxMap;
+      // reading in the binary file
+      std::vector<std::string> txpNames;
+      {
+        std::ifstream ctstream(refNamesFile);
+        cereal::BinaryInputArchive contigTableArchive(ctstream);
+        contigTableArchive(txpNames);
+        ctstream.close();
+      }
+
+      std::vector<uint32_t> txpLengths;
+      {
+        std::ifstream ctstream(refLengthFile);
+        cereal::BinaryInputArchive contigTableArchive(ctstream);
+        contigTableArchive(txpLengths);
+        ctstream.close();
+      }
+
+      {
+        std::ifstream hstream(headerFile);
+        cereal::JSONInputArchive headerArchive(hstream);
+        headerArchive( cereal::make_nvp("num_decoys", numberOfDecoys) );
+        headerArchive( cereal::make_nvp("first_decoy_index", firstDecoyIndex) );
+        headerArchive( cereal::make_nvp("k", kSize) );
+        hstream.close();
+      }
+
+      size_t numShort {0};
+      {
+        // kk this is tricky.
+        // firstDecoyIndex is the index of the first decoy *after*
+        // removing short transcripts but the reported mappings are
+        // in full vector i.e. including small transcripts
+        size_t i {0};
+        if (numberOfDecoys > 0) {
+          while ( i-numShort < firstDecoyIndex ) {
+            if (txpLengths[i] <= kSize) { numShort += 1; }
+            txpIdxMap[txpNames[i]].emplace_back(i);
+            i += 1;
+          }
+        } else {
+          for (size_t i=0; i < txpNames.size(); i++) {
+            if (txpLengths[i] <= kSize) { numShort += 1; }
+            txpIdxMap[txpNames[i]].emplace_back(i);
+          } // end-for
+        } // end else
+      } // end block for populating txpIdxMap
+
+      for (auto it: txpIdxMap) {
+        size_t bucketLen = it.second.size();
+        if (bucketLen > 1) { numDupTxps += (bucketLen - 1); }
+      }
+
+      jointLog->info("Found {} transcripts(+{} decoys, +{} short and +{}"
+                     " duplicate names in the index)",
+                     txpNames.size() - numberOfDecoys - numShort - numDupTxps ,
+                     numberOfDecoys, numShort, numDupTxps);
+
+      if (noTgMap){ // mapping the protein name to itself
+        for (size_t i=0; i<txpNames.size(); i++) {
+          geneIdxMap[txpNames[i]] = i;
+          txpToGeneMap[i] = i;
+        }
+      } else {
+        std::ifstream t2gFile(t2gFileName);
+        uint32_t gid, geneCount{0};
+        std::vector<uint32_t> tids;
+        std::string tStr, gStr;
+        if(t2gFile.is_open()) {
+          while( not t2gFile.eof() ) {
+            t2gFile >> tStr >> gStr;
+
+            if(not txpIdxMap.contains(tStr)  ){
+              continue;
+            }
+
+            tids = txpIdxMap[tStr];
+            if (geneIdxMap.contains(gStr)){
+              gid = geneIdxMap[gStr];
+            }
+            else{
+              gid = geneCount;
+              geneIdxMap[gStr] = gid;
+              geneCount++;
+            }
+
+            for (auto tid: tids) {
+              if (txpToGeneMap.find(tid) != txpToGeneMap.end() &&
+                  txpToGeneMap[tid] != gid ) {
+                jointLog->warn("Dual txp to gene map for txp {}", txpNames[tid]);
+              }
+              txpToGeneMap[tid] = gid;
+            }
+          }
+          t2gFile.close();
+        }
+      } // done parsing txp to gene map file
+
+      jointLog->info( "Filled with {} txp to gene entries ", txpToGeneMap.size());
+      for ( auto it: txpIdxMap ) {
+        for (auto tid: it.second) {
+          if (tid < firstDecoyIndex + numShort &&
+              txpToGeneMap.find( tid ) == txpToGeneMap.end() ) {
+            jointLog->error( "ERROR: Can't find gene mapping for : {} w/ index {}",
+                             it.first, tid );
+            jointLog->error( "ERROR: "
+                             "Txp to Gene Map not found for {}"
+                             " transcripts. Exiting",
+                             txpIdxMap.size() - txpToGeneMap.size() - numberOfDecoys
+                             );
+            jointLog->flush();
+            exit(1);
+          }
+        }
+      }
+
+      jointLog->info("Found all transcripts to gene mappings");
+    }
 
     template <typename ProtocolT>
     bool processAlevinOpts(AlevinOpts<ProtocolT>& aopt,
-                           SalmonOpts& sopt,
+                           SalmonOpts& sopt, bool noTgMap,
                            boost::program_options::variables_map& vm){
       namespace bfs = boost::filesystem;
       namespace po = boost::program_options;
@@ -261,6 +476,8 @@ namespace alevin {
       // mark in salmon options that we are running
       // in alevin mode
       sopt.alevinMode = true;
+      sopt.hardFilter = true;
+      if (sopt.initUniform) { aopt.initUniform = true; }
 
       //Create outputDirectory
       aopt.outputDirectory = vm["output"].as<std::string>() + "/alevin";
@@ -300,17 +517,45 @@ namespace alevin {
         }
       }
 
-      if (vm.count("tgMap")){
-        aopt.geneMapFile = vm["tgMap"].as<std::string>();
-        if (!bfs::exists(aopt.geneMapFile)) {
-          fmt::print(stderr,"\nTranscript to Gene Map File {} does not exists\n Exiting Now",
-                     aopt.geneMapFile.string());
+      if (vm.count("vbemPrior")){
+        aopt.vbemPriorFile = vm["vbemPrior"].as<std::string>();
+        aopt.useVBEM = true;
+
+        aopt.vbemNorm = vm["vbemNorm"].as<double>();
+        if (aopt.vbemNorm == 0.0) {
+          fmt::print(stderr,"\nVBEM Normalization Factor not provided");
+          return false;
+        }
+
+        if (!bfs::exists(aopt.vbemPriorFile)) {
+          fmt::print(stderr,"\nVBEM Prior File {} does not exists\n Exiting Now",
+                     aopt.vbemPriorFile.string());
           return false;
         }
       }
-      else{
-        fmt::print(stderr,"\nTranscript to Gene Map File not provided\n Exiting Now");
-        return false;
+
+      if (vm.count("hash")){
+        aopt.bfhFile = vm["hash"].as<std::string>();
+        if (!bfs::exists(aopt.bfhFile)) {
+          fmt::print(stderr,"\nBfh File {} does not exists\n Exiting Now",
+                     aopt.bfhFile.string());
+          return false;
+        }
+      }
+
+      if (not noTgMap) {
+        if (vm.count("tgMap")){
+          aopt.geneMapFile = vm["tgMap"].as<std::string>();
+          if (!bfs::exists(aopt.geneMapFile)) {
+            fmt::print(stderr,"\nTranscript to Gene Map File {} does not exists\n Exiting Now",
+                       aopt.geneMapFile.string());
+            return false;
+          }
+        }
+        else{
+          fmt::print(stderr,"\nTranscript to Gene Map File not provided\n Exiting Now");
+          return false;
+        }
       }
 
       //create logger
@@ -323,27 +568,31 @@ namespace alevin {
 
       aopt.quiet = vm["quiet"].as<bool>();
       aopt.noEM = vm["noem"].as<bool>();
-      aopt.debug = vm["debug"].as<bool>();
-      aopt.useCorrelation = vm["useCorrelation"].as<bool>();
       aopt.noDedup = vm["noDedup"].as<bool>();
       aopt.naiveEqclass = vm["naiveEqclass"].as<bool>();
       aopt.noQuant = vm["noQuant"].as<bool>();
-      aopt.noSoftMap = vm["noSoftMap"].as<bool>();
       aopt.dumpfq = vm["dumpfq"].as<bool>();
-      aopt.nobarcode = vm["noBarcode"].as<bool>();
+      aopt.dumpArborescences = vm["dumpArborescences"].as<bool>();
       aopt.dumpfeatures = vm["dumpFeatures"].as<bool>();
-      aopt.dumpCsvCounts = vm["dumpCsvCounts"].as<bool>();
+      aopt.dumpMtx = vm["dumpMtx"].as<bool>();
       aopt.dumpBarcodeEq = vm["dumpBarcodeEq"].as<bool>();
-      aopt.dumpBarcodeMap = vm["dumpBarcodeMap"].as<bool>();
-      aopt.dumpUmiToolsMap = vm["dumpUmitoolsMap"].as<bool>();
       aopt.dumpBFH = vm["dumpBfh"].as<bool>();
       aopt.dumpUmiGraph = vm["dumpUmiGraph"].as<bool>();
       aopt.trimRight = vm["trimRight"].as<uint32_t>();
       aopt.numBootstraps = vm["numCellBootstraps"].as<uint32_t>();
       aopt.lowRegionMinNumBarcodes = vm["lowRegionMinNumBarcodes"].as<uint32_t>();
       aopt.maxNumBarcodes = vm["maxNumBarcodes"].as<uint32_t>();
+      aopt.freqThreshold = vm["freqThreshold"].as<uint32_t>();
+      aopt.umiEditDistance = vm["umiEditDistance"].as<uint32_t>();
       aopt.forceCells = vm["forceCells"].as<uint32_t>();
       aopt.expectCells = vm["expectCells"].as<uint32_t>();
+
+      if (aopt.umiEditDistance > 4 ) {
+        aopt.jointLog->error("Too high edit distance collapsing {}, expected <= 4",
+                             aopt.umiEditDistance);
+        return false;
+      }
+
       if(vm.count("iupac")){
         aopt.iupac = vm["iupac"].as<std::string>();
       }
@@ -358,6 +607,20 @@ namespace alevin {
         return false;
       }
 
+      aopt.keepCBFraction = vm["keepCBFraction"].as<double>();
+      if ( aopt.keepCBFraction > 0.0 ) {
+        if ( vm.count("whitelist") ) {
+          aopt.jointLog->error("keepCBFraction and whitelist cannot be used together");
+          aopt.jointLog->flush();
+          exit(1);
+        }
+
+        aopt.jointLog->warn("Force Cells to {} fraction of All possible CB."
+                            "This is not recommended way to run the pipeline,"
+                            "and it might slow the pipeline",
+                            aopt.keepCBFraction);
+      } else if (noTgMap) { aopt.keepCBFraction = 1.0;  }
+
       if (not vm.count("threads")) {
         auto tot_cores = std::thread::hardware_concurrency();
         aopt.numThreads = std::max(1, static_cast<int>(tot_cores/4.0));
@@ -369,9 +632,6 @@ namespace alevin {
       }  // things which needs to be updated for salmonOpts
 
       // validate customized options for custom protocol
-      // NOTE : @k3yavi, I tried to clean this up a little bit
-      // because the previous logic was a little complicated.
-      // Please look over the below.
       bool haveCustomEnd = vm.count("end");
       bool haveCustomBC= vm.count("barcodeLength");
       bool haveCustomUMI = vm.count("umiLength");
@@ -396,8 +656,8 @@ namespace alevin {
         uint32_t umiLength = vm["umiLength"].as<uint32_t>();
 
         // validate that BC and UMI lengths are OK
-        uint32_t maxBC{60};
-        uint32_t maxUMI{12};
+        uint32_t maxBC{20};
+        uint32_t maxUMI{20};
         if (barcodeLength < 1 or barcodeLength > maxBC) {
           aopt.jointLog->error("Barcode length ({}) was not in the required length range [1, {}].\n"
                                "Exiting now.", barcodeLength, maxBC);
@@ -474,14 +734,24 @@ namespace alevin {
 
       // enable validate mappings
       sopt.validateMappings = true;
+      sopt.hitFilterPolicyStr = "BOTH";
       bool optionsOK =
         salmon::utils::processQuantOptions(sopt, vm, vm["numBiasSamples"].as<int32_t>());
       if (!vm.count("minScoreFraction")) {
-        sopt.minScoreFraction = alevin::defaults::minScoreFraction;
+        if (noTgMap) {
+          int32_t l = vm["featureLength"].as<size_t>();
+          sopt.minScoreFraction = salmon::utils::compute_1_edit_threshold(l, sopt);
+          aopt.jointLog->info("set CITE-seq minScoreFraction parameter to : {}",
+                              sopt.minScoreFraction);
+        } else {
+          sopt.minScoreFraction = alevin::defaults::minScoreFraction;
+        }
+
+        sopt.consensusSlack = alevin::defaults::consensusSlack;
         sopt.jointLog->info(
-                            "Using default value of {} for minScoreFraction in Alevin",
-                            sopt.minScoreFraction
-                            );
+            "Using default value of {} for minScoreFraction in Alevin\n"
+            "Using default value of {} for consensusSlack in Alevin",
+            sopt.minScoreFraction, sopt.consensusSlack);
       }
 
       if (!optionsOK) {
@@ -503,6 +773,27 @@ namespace alevin {
       return (sequence.length() > 0) and (sequence.find_first_not_of("ACGTacgt") == std::string::npos);
     }
 
+    bool recoverBarcode(std::string& sequence){
+      size_t pos = sequence.find_first_of("Nn");
+      if (pos == std::string::npos) { return false; }
+
+      // Randomly assigning 'A' to first base with 'N'
+      sequence[pos] = 'A';
+      return sequenceCheck(sequence);
+    }
+
+    void readWhitelist(bfs::path& filePath,
+                       TrueBcsT& trueBarcodes) {
+      std::ifstream whiteFile(filePath.string());
+      std::string whtBc;
+      if(whiteFile.is_open()) {
+        while(getline(whiteFile, whtBc)) {
+          trueBarcodes.insert(whtBc);
+        }
+        whiteFile.close();
+      }
+    }
+
     bool checkSetCoverage(std::vector<std::vector<uint32_t>>& tgroup,
                           std::vector<uint32_t> txps){
       // make sparse hash set for constant membership check
@@ -581,35 +872,43 @@ namespace alevin {
 
     template
     bool processAlevinOpts(AlevinOpts<apt::DropSeq>& aopt,
-                           SalmonOpts& sopt,
+                           SalmonOpts& sopt, bool noTgMap,
+                           boost::program_options::variables_map& vm);
+    template
+    bool processAlevinOpts(AlevinOpts<apt::CITESeq>& aopt,
+                           SalmonOpts& sopt, bool noTgMap,
                            boost::program_options::variables_map& vm);
     template
     bool processAlevinOpts(AlevinOpts<apt::InDrop>& aopt,
-                           SalmonOpts& sopt,
+                           SalmonOpts& sopt, bool noTgMap,
                            boost::program_options::variables_map& vm);
     template
     bool processAlevinOpts(AlevinOpts<apt::ChromiumV3>& aopt,
-                           SalmonOpts& sopt,
+                           SalmonOpts& sopt, bool noTgMap,
                            boost::program_options::variables_map& vm);
     template
     bool processAlevinOpts(AlevinOpts<apt::Chromium>& aopt,
-                           SalmonOpts& sopt,
+                           SalmonOpts& sopt, bool noTgMap,
                            boost::program_options::variables_map& vm);
     template
     bool processAlevinOpts(AlevinOpts<apt::Gemcode>& aopt,
-                           SalmonOpts& sopt,
+                           SalmonOpts& sopt, bool noTgMap,
                            boost::program_options::variables_map& vm);
     template
     bool processAlevinOpts(AlevinOpts<apt::Custom>& aopt,
-                           SalmonOpts& sopt,
+                           SalmonOpts& sopt, bool noTgMap,
                            boost::program_options::variables_map& vm);
     template
     bool processAlevinOpts(AlevinOpts<apt::CELSeq>& aopt,
-                           SalmonOpts& sopt,
+                           SalmonOpts& sopt, bool noTgMap,
                            boost::program_options::variables_map& vm);
     template
     bool processAlevinOpts(AlevinOpts<apt::CELSeq2>& aopt,
-                           SalmonOpts& sopt,
+                           SalmonOpts& sopt, bool noTgMap,
+                           boost::program_options::variables_map& vm);
+    template
+    bool processAlevinOpts(AlevinOpts<apt::QuartzSeq2>& aopt,
+                           SalmonOpts& sopt, bool noTgMap,
                            boost::program_options::variables_map& vm);
   }
 }
