@@ -17,7 +17,9 @@ namespace salmon {
       ("seqBias", po::bool_switch(&(sopt.biasCorrect))->default_value(salmon::defaults::seqBiasCorrect),
        "Perform sequence-specific bias correction.")
       ("gcBias", po::bool_switch(&(sopt.gcBiasCorrect))->default_value(salmon::defaults::gcBiasCorrect),
-       "[beta for single-end reads] Perform fragment GC bias correction")
+       "[beta for single-end reads] Perform fragment GC bias correction.")
+      ("posBias", po::bool_switch(&(sopt.posBiasCorrect))->default_value(salmon::defaults::posBiasCorrect),
+        "Perform positional bias correction.")
       ("threads,p",
        po::value<uint32_t>(&(sopt.numThreads))->default_value(sopt.numThreads),
        "The number of threads to use concurrently.")
@@ -55,10 +57,15 @@ namespace salmon {
        "transcript identifier and the \"gene_id\" is assumed to contain the "
        "corresponding "
        "gene identifier.")
+      ("auxTargetFile", po::value(&(sopt.auxTargetFile))->default_value(salmon::defaults::auxTargetFile),
+      "A file containing a list of \"auxiliary\" targets.  These are valid targets (i.e., not decoys) to "
+      "which fragments are allowed to map and be assigned, and which will be quantified, but for which "
+      "auxiliary models like sequence-specific and fragment-GC bias correction should not be applied."
+      )
       ("meta", po::bool_switch(&(sopt.meta))->default_value(salmon::defaults::metaMode),
        "If you're using Salmon on a metagenomic dataset, consider setting this "
-       "flag to disable parts of the "
-       "abundance estimation model that make less sense for metagenomic data.");
+       "flag to disable parts of the abundance estimation model that make less "
+       "sense for metagenomic data.");
 
     // use rich eq classes by default
     sopt.noRichEqClasses = salmon::defaults::noRichEqClasses;
@@ -77,72 +84,156 @@ namespace salmon {
     mapspec.add_options()
       ("discardOrphansQuasi",
        po::bool_switch(&(sopt.discardOrphansQuasi))->default_value(salmon::defaults::discardOrphansQuasi),
-       "[Quasi-mapping mode only] : Discard orphan mappings in quasi-mapping "
+       "[selective-alignment mode only] : Discard orphan mappings in quasi-mapping "
        "mode.  If this flag is passed "
        "then only paired mappings will be considered toward quantification "
        "estimates.  The default behavior is "
        "to consider orphan mappings if no valid paired mappings exist.  This "
        "flag is independent of the option to "
        "write the orphaned mappings to file (--writeOrphanLinks).")
+       /*
+      ("noSA",
+       po::bool_switch(&(sopt.disableSA))->default_value(salmon::defaults::disableSA),
+       "[Not currently supported] : Disable selective-alignment in favor of basic quasi-mapping. "
+       "If this flag is passed, selective-alignment and alignment scoring of reads will be disabled."
+       )*/
       ("validateMappings",
        po::bool_switch(&(sopt.validateMappings))->default_value(salmon::defaults::validateMappings),
-       "[Quasi-mapping mode only] : Validate mappings using alignment-based verifcation. "
-       "If this flag is passed, quasi-mappings will be validated to ensure that they could give "
-       "rise to a reasonable alignment before they are further used for quantification.")
+       "[*deprecated* (no effect; selective-alignment is the default)]")
       ("consensusSlack",
-       po::value<int32_t>(&(sopt.consensusSlack))->default_value(salmon::defaults::consensusSlack),
-       "[Quasi-mapping mode only] : The amount of slack allowed in the quasi-mapping consensus "
+       po::value<float>(&(sopt.consensusSlack))->default_value(salmon::defaults::consensusSlack),
+       "[selective-alignment mode only] : The amount of slack allowed in the quasi-mapping consensus "
        "mechanism.  Normally, a transcript must cover all hits to be considered for mapping.  "
-       "If this is set to a value, X, greater than 0, then a transcript can fail to cover up to "
-       "X hits before it is discounted as a mapping candidate.  The default value of this option "
-       "is 1 if --validateMappings is given and 0 otherwise."
+       "If this is set to a fraction, X, greater than 0 (and in [0,1)), then a transcript can fail to cover up to "
+       "(100 * X)% of the hits before it is discounted as a mapping candidate.  The default value of this option "
+       "is 0.2 if --validateMappings is given and 0 otherwise."
        )
+      ("scoreExp", 
+      po::value<double>(&sopt.scoreExp)->default_value(salmon::defaults::scoreExp),
+      "[selective-alignment mode only] : The factor by which sub-optimal alignment scores are "
+      "downweighted to produce a probability.  If the best alignment score for the current read is S, and the score "
+      "for a particular alignment is w, then the probability will be computed porportional to exp( - scoreExp * (S-w) )."
+      )
       ("minScoreFraction",
        po::value<double>(&sopt.minScoreFraction),
-       "[Quasi-mapping mode (w / mapping validation) only] : The fraction of the optimal possible alignment score that a "
+       "[selective-alignment mode only] : The fraction of the optimal possible alignment score that a "
        "mapping must achieve in order to be considered \"valid\" --- should be in (0,1].\n"
-       "Salmon Default 0.65 and Alevin Default 0.8"
-       )
-      ("maxMMPExtension",
-       po::value<int32_t>(&sopt.maxMMPExtension)->default_value(salmon::defaults::maxMMPExtension),
-       "[Quasi-mapping mode (w / mapping validation) only] : Sets the maximum allowable MMP extension when collecting "
-       "suffix array intervals to be used in chaining.  This prevents MMPs from becoming too long, and potentially masking "
-       "intervals that would uncover other good quasi-mappings for the read.  This heuristic mimics the idea of the "
-       "maximum mappable safe prefix (MMSP) in selective alignment.  Setting a smaller value will potentially allow "
-       "for more sensitive, but slower, mapping."
+       "Salmon Default 0.65 and Alevin Default 0.87"
        )
+      ("mismatchSeedSkip",
+       po::value<uint32_t>(&sopt.mismatchSeedSkip)->default_value(salmon::defaults::mismatchSeedSkip),
+       "[selective-alignment mode only] : After a k-mer hit is extended to a uni-MEM, the uni-MEM extension "
+       "can terminate for one of 3 reasons; the end of the read, the end of the unitig, or a mismatch.  If the "
+       "extension ends because of a mismatch, this is likely the result of a sequencing error.  To avoid looking "
+       "up many k-mers that will likely fail to be located in the index, the search procedure skips by a factor "
+       "of mismatchSeedSkip until it either (1) finds another match or (2) is k-bases past the mismatch position. "
+       "This value controls that skip length.  A smaller value can increase sensitivity, while a larger value "
+       "can speed up seeding."
+      )
+      ("disableChainingHeuristic",
+      po::bool_switch(&(sopt.disableChainingHeuristic))->default_value(salmon::defaults::disableChainingHeuristic),
+      "[selective-alignment mode only] : By default, the heuristic of (Li 2018) is implemented, which terminates "
+      "the chaining DP once a given number of valid backpointers are found.  This speeds up the seed (MEM) "
+      "chaining step, but may result in sub-optimal chains in complex situations (e.g. sequences with many repeats and "
+      "overlapping repeats).  Passing this flag will disable the chaining heuristic, and perform the full chaining "
+      "dynamic program, guaranteeing the optimal chain is found in this step."
+      )
+      ("decoyThreshold",
+       po::value<double>(&sopt.decoyThreshold)->default_value(salmon::defaults::decoyThreshold),
+       "[selective-alignment mode only] : For an alignemnt to an annotated transcript to be considered invalid, it must have an alignment "
+       "score < (decoyThreshold * bestDecoyScore).  A value of 1.0 means that any alignment strictly worse than "
+       "the best decoy alignment will be discarded.  A smaller value will allow reads to be allocated to transcripts "
+       "even if they strictly align better to the decoy sequence."
+      )
       ("ma",
        po::value<int16_t>(&sopt.matchScore)->default_value(salmon::defaults::matchScore),
-       "[Quasi-mapping mode (w / mapping validation) only] : The value given to a match between read and reference nucleotides "
+       "[selective-alignment mode only] : The value given to a match between read and reference nucleotides "
        "in an alignment."
        )
       ("mp",
        po::value<int16_t>(&sopt.mismatchPenalty)->default_value(salmon::defaults::mismatchPenalty),
-       "[Quasi-mapping mode (w / mapping validation) only] : The value given to a mis-match between read and reference nucleotides "
+       "[selective-alignment mode only] : The value given to a mis-match between read and reference nucleotides "
        "in an alignment."
        )
       ("go",
        po::value<int16_t>(&sopt.gapOpenPenalty)->default_value(salmon::defaults::gapOpenPenalty),
-       "[Quasi-mapping mode (w / mapping validation) only] : The value given to a gap opening in an alignment."
+       "[selective-alignment mode only] : The value given to a gap opening in an alignment."
        )
       ("ge",
        po::value<int16_t>(&sopt.gapExtendPenalty)->default_value(salmon::defaults::gapExtendPenalty),
-       "[Quasi-mapping mode (w / mapping validation) only] : The value given to a gap extension in an alignment."
+       "[selective-alignment mode only] : The value given to a gap extension in an alignment."
+       )
+      ("bandwidth",
+       po::value<int32_t>(&sopt.dpBandwidth)->default_value(salmon::defaults::dpBandwidth),
+       "[selective-alignment mode only] : The value used for the bandwidth passed to ksw2.  A smaller "
+       "bandwidth can make the alignment verification run more quickly, but could possibly miss valid alignments."
+       )
+      ("allowDovetail",
+       po::bool_switch(&(sopt.allowDovetail))->default_value(salmon::defaults::allowDovetail),
+       "[selective-alignment mode only] : allow dovetailing mappings."
+       )
+      ("recoverOrphans",
+       po::bool_switch(&(sopt.recoverOrphans))->default_value(salmon::defaults::recoverOrphans),
+       "[selective-alignment mode only] : Attempt to recover the mates of orphaned reads. This uses edlib for "
+       "orphan recovery, and so introduces some computational overhead, but it can improve sensitivity."
+       )
+      ("mimicBT2", // horrible flag name, think of something better
+       po::bool_switch(&(sopt.mimicBT2))->default_value(salmon::defaults::mimicBT2),
+       "[selective-alignment mode only] : Set flags to mimic parameters similar to "
+       "Bowtie2 with --no-discordant and --no-mixed flags.  This increases disallows dovetailing reads, and "
+       "discards orphans. Note, this does not impose the very strict parameters assumed by RSEM+Bowtie2, "
+       "like gapless alignments.  For that behavior, use the --mimiStrictBT2 flag below."
        )
       ("mimicStrictBT2", // horrible flag name, think of something better
        po::bool_switch(&(sopt.mimicStrictBT2))->default_value(salmon::defaults::mimicStrictBT2),
-       "[Quasi-mapping mode (w / mapping validation) only] : Set flags to mimic the very strict parameters used by "
+       "[selective-alignment mode only] : Set flags to mimic the very strict parameters used by "
        "RSEM+Bowtie2.  This increases --minScoreFraction to 0.8, disallows dovetailing reads, "
        "discards orphans, and disallows gaps in alignments."
        )
-      ("allowOrphansFMD",
-       po::bool_switch(&(sopt.allowOrphans))->default_value(salmon::defaults::allowOrphansFMD),
-       "[FMD-mapping mode only] : Consider orphaned reads as valid hits when "
-       "performing lightweight-alignment.  This option will increase "
-       "sensitivity (allow more reads to map and "
-       "more transcripts to be detected), but may decrease specificity as "
-       "orphaned alignments are more likely "
-       "to be spurious.")
+      ("softclip", 
+       po::bool_switch(&(sopt.softclip))->default_value(salmon::defaults::softclip),
+       "[selective-alignment mode only (experimental)] : Allos soft-clipping of reads during selective-alignment. If this "
+       "option is provided, then regions at the beginning or end of the read can be withheld from alignment "
+       "without any effect on the resulting score (i.e. neither adding nor removing from the score).  This "
+       "will drastically reduce the penalty if there are mismatches at the beginning or end of the read "
+       "due to e.g. low-quality bases or adapters.  NOTE: Even with soft-clipping enabled, the read must still "
+       "achieve a score of at least minScoreFraction * maximum achievable score, where the maximum achievable "
+       "score is computed based on the full (un-clipped) read length."
+      )
+      ("softclipOverhangs", 
+       po::bool_switch(&(sopt.softclipOverhangs))->default_value(salmon::defaults::softclipOverhangs),
+       "[selective-alignment mode only] : Allow soft-clipping of reads that overhang the beginning or ends "
+       "of the transcript.  In this case, the overhaning section of the read will simply be unaligned, and "
+       "will not contribute or detract from the alignment score.  The default policy is to force an end-to-end "
+       "alignemnt of the entire read, so that overhanings will result in some deletion of nucleotides from the "
+       "read."
+       )
+      ("fullLengthAlignment", 
+       po::bool_switch(&(sopt.fullLengthAlignment))->default_value(salmon::defaults::fullLengthAlignment),
+       "[selective-alignment mode only] : Perform selective alignment over the full length of the read, beginning "
+       "from the (approximate) initial mapping location and using extension alignment.  This is in contrast with the "
+       "default behavior which is to only perform alignment between the MEMs in the optimal chain (and before the "
+       "first and after the last MEM if applicable).  The default strategy forces the MEMs to belong to the alignment, "
+       "but has the benefit that it can discover indels prior to the first hit shared between the read and reference. Except in "
+       "very rare circumstances, the default mode should be more accurate."
+       )
+      ("hardFilter",
+       po::bool_switch(&(sopt.hardFilter))->default_value(salmon::defaults::hardFilter),
+       "[selective-alignemnt mode only] : Instead of weighting mappings by their alignment score, "
+       "this flag will discard any mappings with sub-optimal alignment score.  The default option of soft-filtering "
+       "(i.e. weighting mappings by their alignment score) usually yields slightly more accurate abundance estimates "
+       "but this flag may be desirable if you want more accurate 'naive' equivalence classes, rather "
+       "than range factorized equivalence classes."
+       )
+       ("minAlnProb",
+       po::value<double>(&(sopt.minAlnProb))->default_value(salmon::defaults::minAlnProb),
+       "[selective-alignment mode only] : Any mapping whose alignment probability (as computed by "
+       "P(aln) = exp(-scoreExp * difference from best mapping score) is less than minAlnProb will "
+       "not be considered as a valid alignment for this read.  The goal of this flag is to remove "
+       "very low probability alignments that are unlikely to have any non-trivial effect on the "
+       "final quantifications.  Filtering such alignments reduces the number of variables that need "
+       "to be considered and can result in slightly faster inference and 'cleaner' equivalence classes."
+       )
       ("writeMappings,z", po::value<string>(&sopt.qmFileName)
        ->default_value(salmon::defaults::quasiMappingDefaultFile)
        ->implicit_value(salmon::defaults::quasiMappingImplicitFile),
@@ -151,37 +242,17 @@ namespace salmon {
        "format.  By default, output will be directed to "
        "stdout, but an alternative file name can be "
        "provided instead.")
-      ("consistentHits,c",
-       po::bool_switch(&(sopt.consistentHits))->default_value(salmon::defaults::consistentHits),
-       "Force hits gathered during "
-       "quasi-mapping to be \"consistent\" (i.e. co-linear and "
-       "approximately the right distance apart).")
-      ("strictIntersect",
-       po::bool_switch(&(sopt.strictIntersect))->default_value(salmon::defaults::strictIntersect),
-       "Modifies how orphans are "
-       "assigned.  When this flag is set, if the intersection of the "
-       "quasi-mappings for the left and right "
-       "is empty, then all mappings for the left and all mappings for the "
-       "right read are reported as orphaned "
-       "quasi-mappings")
-      ("fasterMapping",
-       po::bool_switch(&(sopt.fasterMapping))->default_value(salmon::defaults::fasterMapping),
-       "[Developer]: Disables some extra checks during quasi-mapping. This "
-       "may make mapping a "
-       "little bit faster at the potential cost of returning too many "
-       "mappings (i.e. some sub-optimal mappings) "
-       "for certain reads.")
-      ("quasiCoverage,x",
-       po::value<double>(&(sopt.quasiCoverage))->default_value(salmon::defaults::quasiCoverage),
-       "[Experimental]: The fraction of the read that must be covered by "
-       "MMPs (of length >= 31) if "
-       "this read is to be considered as \"mapped\".  This may help to "
-       "avoid \"spurious\" mappings. "
-       "A value of 0 (the default) denotes no coverage threshold (a single "
-       "31-mer can yield a mapping).  "
-       "Since coverage by exact matching, large, MMPs is a rather strict "
-       "condition, this value should likely "
-       "be set to something low, if used.");
+      ("hitFilterPolicy",
+       po::value<string>(&sopt.hitFilterPolicyStr)->default_value(salmon::defaults::hitFilterPolicyStr),
+       "[selective-alignment mode only] : Determines the policy by which hits are filtered in selective alignment.  Filtering hits after "
+       "chaining (the default) is more sensitive, but more computationally intensive, because it performs "
+       "the chaining dynamic program for all hits.  Filtering before chaining is faster, but some true hits "
+       "may be missed.  The options are BEFORE, AFTER, BOTH and NONE."
+       );
+
+      sopt.disableSA = salmon::defaults::disableSA;
+      sopt.fasterMapping = salmon::defaults::fasterMapping;
+      
       return mapspec;
   }
 
@@ -215,14 +286,16 @@ namespace salmon {
     alignin.add_options()
     ("discardOrphans",
        po::bool_switch(&(sopt.discardOrphansAln))->default_value(salmon::defaults::discardOrphansAln),
-       "[Alignment-based mode only] : Discard orphan alignments in the input "
+       "[alignment-based mode only] : Discard orphan alignments in the input "
        ".  If this flag is passed, then only paired alignments will be "
        "considered toward quantification estimates.  The default behavior is "
        "to consider orphan alignments if no valid paired mappings exist.")
     ("libType,l", po::value<std::string>()->required(), "Format string describing the library type")
-    ("alignments,a", po::value<vector<string>>()->multitoken()->required(),
+    ("alignments,a", po::value<vector<string>>()->multitoken(),
      "input alignment (BAM) file(s).")
-    ("targets,t", po::value<std::string>()->required(),
+    ("eqclasses,e", po::value<string>(),
+     "input salmon weighted equivalence class file.")
+    ("targets,t", po::value<std::string>(),
      "FASTA format file containing target transcripts.");
 
     return alignin;
@@ -241,53 +314,24 @@ namespace salmon {
       (
        "dumpBarcodeEq", po::bool_switch()->default_value(alevin::defaults::dumpBarcodeEq),
        "Dump JointEqClas with umi-barcode count.")
-      (
-       "dumpUmitoolsMap", po::bool_switch()->default_value(alevin::defaults::dumpUMIToolsMap),
-       "Dump umi_tools readable whitelist map for downstream analysis.")
-      (
-       "dumpBarcodeMap", po::bool_switch()->default_value(alevin::defaults::dumpBarcodeMap),
-       "Dump BarcodeMap for downstream analysis.")
       (
        "iupac,u",po::value<std::string>(),
        "<Deprecated>iupac code for cell-level barcodes.")
       (
-       "end",po::value<uint32_t>(),
-       "Cell-Barcodes end (5 or 3) location in the read sequence from where barcode has to"
-       "be extracted. (end, umiLength, barcodeLength)"
-       " should all be provided if using this option")
+       "vbemPrior", po::value<std::string>(),
+       "a mtx file containing VBEM priors")
       (
-       "umiLength",po::value<uint32_t>(),
-       "umi length Parameter for unknown protocol. (end, umiLength, barcodeLength)"
-       " should all be provided if using this option")
-      (
-       "barcodeLength",po::value<uint32_t>(),
-       "umi length Parameter for unknown protocol. (end, umiLength, barcodeLength)"
-       " should all be provided if using this option")
-      (
-       "noem",po::bool_switch()->default_value(alevin::defaults::noEM),
-       "do not run em")
-      (
-       "noBarcode",po::bool_switch()->default_value(alevin::defaults::noBarcode),
-       "this flag should be used when there is no barcode i.e. only one cell deduplication.")
+       "vbemNorm",po::value<double>()->default_value(alevin::defaults::vbemNorm),
+       "Variational Bayesian global normalization factor. Usually the total number of deduplicated UMIs.")
       (
        "trimRight",po::value<uint32_t>()->default_value(alevin::defaults::trimRight),
        "The number of bases to trim off the 5' (right) end of the read seequence.")
-      (
-       "noSoftMap", po::bool_switch()->default_value(alevin::defaults::noSoftMap),
-       "Don't use soft-assignment for quant instead do hard-assignment.")
       (
        "naiveEqclass", po::bool_switch()->default_value(alevin::defaults::naiveEqclass),
        "Run naive per equivalence class deduplication, generating only total number of UMIs")
       (
        "noDedup", po::bool_switch()->default_value(alevin::defaults::noDedup),
-       "Stops the pipeline after CB sequence correction and quasi-mapping reads.")
-      (
-       "debug", po::bool_switch()->default_value(alevin::defaults::debug),
-       "Enabling this mode mode will try to ignore segfaults based on no whitelist"
-       " mapping or no whitelist deduplicated count")
-      (
-       "freqThreshold",po::value<uint32_t>(),
-       "threshold for the frequency of the barcodes");
+       "Stops the pipeline after CB sequence correction and quasi-mapping reads.");
     return alevindevs;
   }
 
@@ -303,6 +347,9 @@ namespace salmon {
        "The number of threads to use concurrently.")
       (
        "tgMap", po::value<std::string>(), "transcript to gene map tsv file")
+      (
+       "hash", po::value<std::string>(), "Secondary input point for Alevin "
+       "using Big freaking Hash (bfh.txt) file. Works Only with --chromium")
       (
        "dropseq", po::bool_switch()->default_value(alevin::defaults::isDropseq),
        "Use DropSeq Single Cell protocol for the library")
@@ -315,15 +362,27 @@ namespace salmon {
       (
        "gemcode", po::bool_switch()->default_value(alevin::defaults::isGemcode),
        "Use 10x gemcode v1 Single Cell protocol for the library.")
+      (
+       "citeseq", po::bool_switch()->default_value(alevin::defaults::isCITESeq),
+       "Use CITESeq Single Cell protocol for the library, 16 CB, 12 UMI and features.")
       (
        "celseq", po::bool_switch()->default_value(alevin::defaults::isCELSeq),
        "Use CEL-Seq Single Cell protocol for the library.")
       (
        "celseq2", po::bool_switch()->default_value(alevin::defaults::isCELSeq2),
        "Use CEL-Seq2 Single Cell protocol for the library.")
+      (
+       "quartzseq2", po::bool_switch()->default_value(alevin::defaults::isQuartzSeq2),
+       "Use Quartz-Seq2 v3.2 Single Cell protocol for the library assumes 15 length barcode and 8 length UMI.")
       (
        "whitelist", po::value<std::string>(),
        "File containing white-list barcodes")
+      (
+       "featureStart", po::value<size_t>(),
+       "This flag should be used with citeseq and specifies the starting index of the feature barcode on Read2.")
+      (
+       "featureLength", po::value<size_t>(),
+       "This flag should be used with citeseq and specifies the length of the feature barcode.")
       (
        "noQuant", po::bool_switch()->default_value(alevin::defaults::noQuant),
        "Don't run downstream barcode-salmon model.")
@@ -344,9 +403,31 @@ namespace salmon {
        "rrna", po::value<std::string>(),
        "path to a file containing ribosomal RNA, one per line")
       (
-       "useCorrelation", po::bool_switch()->default_value(alevin::defaults::useCorrelation),
-       "Use pair-wise pearson correlation with True barcodes as a"
-       " feature for white-list creation.")
+       "keepCBFraction", po::value<double>()->default_value(alevin::defaults::keepCBFraction),
+       "fraction of CB to keep, value must be in range (0,1], use 1 to quantify all CB."
+       )
+      (
+       "end",po::value<uint32_t>(),
+       "Cell-Barcodes end (5 or 3) location in the read sequence from where barcode has to"
+       "be extracted. (end, umiLength, barcodeLength)"
+       " should all be provided if using this option")
+      (
+       "umiLength",po::value<uint32_t>(),
+       "umi length Parameter for unknown protocol. (end, umiLength, barcodeLength)"
+       " should all be provided if using this option")
+      (
+       "barcodeLength",po::value<uint32_t>(),
+       "umi length Parameter for unknown protocol. (end, umiLength, barcodeLength)"
+       " should all be provided if using this option")
+      (
+       "noem",po::bool_switch()->default_value(alevin::defaults::noEM),
+       "do not run em")
+      (
+       "freqThreshold", po::value<uint32_t>()->default_value(alevin::defaults::freqThreshold),
+       "threshold for the frequency of the barcodes")
+      (
+       "umiEditDistance", po::value<uint32_t>()->default_value(alevin::defaults::umiEditDistance),
+       "Maximum allowble edit distance to collapse UMIs, Expect delay in running time if != 1")
       (
        "dumpfq", po::bool_switch()->default_value(alevin::defaults::dumpFQ),
        "Dump barcode modified fastq file for downstream analysis by"
@@ -354,6 +435,9 @@ namespace salmon {
       (
        "dumpBfh", po::bool_switch()->default_value(alevin::defaults::dumpBFH),
        "dump the big hash with all the barcodes and the UMI sequence.")
+      (
+       "dumpArborescences", po::bool_switch()->default_value(alevin::defaults::dumpArborescences),
+       "dump the gene-v-cell matrix for the total number of fragments used in the UMI deduplicaiton.")
       (
        "dumpUmiGraph", po::bool_switch()->default_value(alevin::defaults::dumpUmiGraph),
        "dump the per cell level Umi Graph.")
@@ -361,8 +445,8 @@ namespace salmon {
        "dumpFeatures", po::bool_switch()->default_value(alevin::defaults::dumpFeatures),
        "Dump features for whitelist and downstream analysis.")
       (
-       "dumpCsvCounts", po::bool_switch()->default_value(alevin::defaults::dumpCSVCounts),
-       "Dump cell v transcripts count matrix in csv format.")
+       "dumpMtx", po::bool_switch()->default_value(alevin::defaults::dumpMtx),
+       "Dump cell v transcripts count matrix in sparse mtx format.")
       (
        "lowRegionMinNumBarcodes", po::value<uint32_t>()->default_value(alevin::defaults::lowRegionMinNumBarcodes),
        "Minimum Number of CB to use for learning Low confidence region (Default: 200).")
@@ -423,6 +507,14 @@ namespace salmon {
        "\'|\' character.  These reduced names will be used in "
        "the output and when looking for these transcripts in a gene to "
        "transcript GTF.")
+      ("scoreExp", 
+      po::value<double>(&sopt.scoreExp)->default_value(salmon::defaults::scoreExp),
+      "The factor by which sub-optimal alignment scores are "
+      "downweighted to produce a probability.  If the best alignment score for the current read is S, and the score "
+      "for a particular alignment is w, then the probability will be computed porportional to exp( - scoreExp * (S-w) ). "
+      "NOTE: This flag only has an effect if you are parsing alignments produced by salmon itself (i.e. pufferfish or "
+      "RapMap in selective-alignment mode)."
+      )
       (
        "mappingCacheMemoryLimit",
        po::value<uint32_t>(&(sopt.mappingCacheMemoryLimit))
@@ -456,14 +548,20 @@ namespace salmon {
        "The sub-directory of the quantification directory where auxiliary "
        "information "
        "e.g. bootstraps, bias parameters, etc. will be written.")
+      ("skipQuant", po::bool_switch(&(sopt.skipQuant))->default_value(salmon::defaults::skipQuant),
+       "Skip performing the actual transcript quantification (including any Gibbs sampling or bootstrapping)."
+       )
       ("dumpEq", po::bool_switch(&(sopt.dumpEq))->default_value(salmon::defaults::dumpEq),
-       "Dump the equivalence class counts "
-       "that were computed during quasi-mapping")
+       "Dump the simple equivalence class counts "
+       "that were computed during mapping or alignment.")
       ("dumpEqWeights,d",
        po::bool_switch(&(sopt.dumpEqWeights))->default_value(salmon::defaults::dumpEqWeights),
-       "Includes \"rich\" equivlance class weights in the output when "
-       "equivalence "
-       "class information is being dumped to file.")
+       "Dump conditional probabilities associated with transcripts when "
+       "equivalence class information is being dumped to file. Note, this will "
+       "dump the factorization that is actually used by salmon's offline phase "
+       "for inference.  If you are using range-factorized equivalence classes (the default) "
+       "then the same transcript set may appear multiple times with different associated "
+       "conditional probabilities.")
       ("minAssignedFrags",
        po::value<std::uint64_t>(&(sopt.minRequiredFrags))->default_value(salmon::defaults::minAssignedFrags),
        "The minimum number of fragments that must be assigned to the "
@@ -488,10 +586,10 @@ namespace salmon {
        "The maximum fragment length to consider when building the empirical "
        "distribution")
       ("fldMean",
-       po::value<size_t>(&(sopt.fragLenDistPriorMean))->default_value(salmon::defaults::fragLenPriorMean),
+       po::value<double>(&(sopt.fragLenDistPriorMean))->default_value(salmon::defaults::fragLenPriorMean),
        "The mean used in the fragment length distribution prior")
       ("fldSD",
-       po::value<size_t>(&(sopt.fragLenDistPriorSD))->default_value(salmon::defaults::fragLenPriorSD),
+       po::value<double>(&(sopt.fragLenDistPriorSD))->default_value(salmon::defaults::fragLenPriorSD),
        "The standard deviation used in the fragment length distribution "
        "prior")
       ("forgettingFactor,f",
@@ -506,6 +604,9 @@ namespace salmon {
        po::bool_switch(&(sopt.initUniform))->default_value(salmon::defaults::initUniform),
        "initialize the offline inference with uniform parameters, rather "
        "than seeding with online parameters.")
+      ("maxOccsPerHit",
+       po::value<uint32_t>(&(sopt.maxOccsPerHit))->default_value(salmon::defaults::maxOccsPerHit),
+       "When collecting \"hits\" (MEMs), hits having more than maxOccsPerHit occurrences won't be considered.")
       ("maxReadOcc,w",
        po::value<uint32_t>(&(sopt.maxReadOccs))->default_value(salmon::defaults::maxReadOccs),
        "Reads \"mapping\" to more than this many places won't be "
@@ -529,6 +630,12 @@ namespace salmon {
        "from a transcript.  If this flag is passed in, the "
        "fragment length distribution is not taken "
        "into account when computing this probability.")
+      ("noSingleFragProb",
+       po::bool_switch(&(sopt.noSingleFragProb))->default_value(salmon::defaults::noSingleFragProb),
+       "Disables the estimation of an associated fragment length probability for single-end reads or for "
+       "orphaned mappings in paired-end libraries.  The default behavior is to consider the probability of "
+       "all possible fragment lengths associated with the retained mapping.  Enabling this flag (i.e. turning this "
+       "default behavior off) will simply not attempt to estimate a fragment length probability in such cases.")
       ("noFragLengthDist",
        po::bool_switch(&(sopt.noFragLengthDist))->default_value(salmon::defaults::noFragLengthDist),
        "[experimental] : "
@@ -582,10 +689,11 @@ namespace salmon {
        "the conditional probabilities with which fragments are generated "
        "from different transcripts.  This is a more "
        "fine-grained factorization than the normal rich equivalence "
-       "classes.  The default value (0) corresponds to "
-       "the standard rich equivalence classes, and larger values imply a "
-       "more fine-grained factorization.  If range factorization "
-       "is enabled, a common value to select for this parameter is 4.")
+       "classes.  The default value (4) corresponds to "
+       "the default used in Zakeri et al. 2017 (doi: 10.1093/bioinformatics/btx262), "
+       "and larger values imply a more fine-grained factorization.  If range factorization "
+       "is enabled, a common value to select for this parameter is 4. A value of "
+       "0 signifies the use of basic rich equivalence classes.")
       ("numGibbsSamples",
        po::value<uint32_t>(&(sopt.numGibbsSamples))->default_value(salmon::defaults::numGibbsSamples),
        "Number of Gibbs sampling rounds to "
@@ -618,14 +726,22 @@ namespace salmon {
        "be interpreted as a transcript-level prior (i.e. each transcript "
        "will "
        "be given a prior read count of this value)")
+      ("perNucleotidePrior", po::bool_switch(&(sopt.perNucleotidePrior))->default_value(salmon::defaults::perNucleotidePrior),
+       "The "
+       "prior (either the default or the argument provided via --vbPrior) "
+       "will "
+       "be interpreted as a nucleotide-level prior (i.e. each nucleotide "
+       "will "
+       "be given a prior read count of this value)")
       ("sigDigits", po::value<uint32_t>(&(sopt.sigDigits))->default_value(salmon::defaults::sigDigits),
        "The number of significant digits to write when outputting the EffectiveLength and NumReads columns")
       ("vbPrior", po::value<double>(&(sopt.vbPrior))->default_value(salmon::defaults::vbPrior),
        "The prior that will be used in the VBEM algorithm.  This is "
        "interpreted "
-       "as a per-nucleotide prior, unless the --perTranscriptPrior flag "
-       "is also given, in which case this is used as a transcript-level "
-       "prior")
+       "as a per-transcript prior, unless the --perNucleotidePrior flag "
+       "is also given.  If the --perNucleotidePrior flag is given, this is used as a nucleotide-level "
+       "prior.  If the default is used, it will be divided by 1000 before being used as a nucleotide-level "
+       "prior, i.e. the default per-nucleotide prior will be 1e-5.")
       ("writeOrphanLinks",
        po::bool_switch(&(sopt.writeOrphanLinks))->default_value(salmon::defaults::writeOrphanLinks),
        "Write the transcripts that are linked by orphaned reads.")
@@ -665,8 +781,6 @@ namespace salmon {
     po::options_description testing("\n"
                                     "testing options");
     testing.add_options()
-      ("posBias", po::bool_switch(&(sopt.posBiasCorrect))->default_value(salmon::defaults::posBiasCorrect),
-                          "[experimental] Perform positional bias correction")
       ("noRichEqClasses",
       po::bool_switch(&(sopt.noRichEqClasses))->default_value(salmon::defaults::noRichEqClasses),
         "[TESTING OPTION]: Disable \"rich\" equivalent classes.  If this flag is "
@@ -680,6 +794,10 @@ namespace salmon {
         "account the "
         "goodness-of-fit of an alignment with the empirical fragment length "
         "distribution")
+      ("disableAlignmentCache",
+      po::bool_switch(&(sopt.disableAlignmentCache))->default_value(salmon::defaults::disableAlignmentCache),
+        "[TESTING OPTION]: Turn of the alignment cache.  This will hurt performance but "
+        "can help debug any issues that might result from caching")
       ("rankEqClasses",
       po::bool_switch(&(sopt.rankEqClasses))->default_value(salmon::defaults::rankEqClasses),
         "[TESTING OPTION]: Keep separate equivalence classes for each distinct "
@@ -693,17 +811,12 @@ namespace salmon {
     return testing;
   }
 
-  po::options_description ProgramOptionsGenerator::getDeprecatedOptions(SalmonOpts& sopt) {
+  po::options_description ProgramOptionsGenerator::getDeprecatedOptions(SalmonOpts& /*sopt*/) {
     using std::string;
     using std::vector;
 
     po::options_description deprecated(
         "\ndeprecated options about which to inform the user");
-    deprecated.add_options()
-      ("useFSPD", po::bool_switch(&(sopt.useFSPD))->default_value(salmon::defaults::useFSPD),
-        "[deprecated] : "
-        "Consider / model non-uniformity in the fragment start positions "
-            "across the transcript.");
     return deprecated;
   }
 
