@@ -24,6 +24,7 @@
 #include <cstdint>
 #include <cstdio>
 #include <cstring>
+#include <cstdlib>
 #include <fstream>
 #include <functional>
 #include <iostream>
@@ -46,7 +47,7 @@
 #include "SalmonConfig.hpp"
 #include "VersionChecker.hpp"
 
-int help(const std::vector<std::string>& opts) { //}int argc, char* argv[]) {
+int help(const std::vector<std::string>& /*opts*/) { 
   fmt::MemoryWriter helpMsg;
   helpMsg.write("salmon v{}\n\n", salmon::version);
   helpMsg.write(
@@ -93,7 +94,7 @@ int dualModeMessage() {
 /**
  * Bonus!
  */
-int salmonSwim(int argc, const char* argv[]) {
+int salmonSwim(int /*argc*/, const char* /*argv*/[]) {
 
   std::cout << R"(
     _____       __
@@ -147,6 +148,7 @@ bibtex:
 int salmonIndex(int argc, const char* argv[]);
 int salmonQuantify(int argc, const char* argv[]);
 int salmonAlignmentQuantify(int argc, const char* argv[]);
+// TODO : PF_INTEGRATION
 int salmonBarcoding(int argc, const char* argv[]);
 int salmonQuantMerge(int argc, const char* argv[]);
 
@@ -208,9 +210,17 @@ int main(int argc, char* argv[]) {
       std::exit(0);
     }
 
-    if (!vm.count("no-version-check")) {
+    const char* no_version_env_ptr = std::getenv("SALMON_NO_VERSION_CHECK");
+    std::string no_version_env = (no_version_env_ptr == nullptr) ? "" : std::string(no_version_env_ptr);
+    std::transform(no_version_env.begin(), no_version_env.end(), no_version_env.begin(), 
+                   [](unsigned char c){ return std::toupper(c); } // correct
+                  );
+    bool skip_version_check = vm.count("no-version-check") or (no_version_env == "1") 
+                              or (no_version_env == "TRUE") or (no_version_env == "T");
+
+    if (!skip_version_check) {
       std::string versionMessage = getVersionMessage();
-      std::cout << versionMessage;
+      std::cerr << versionMessage;
     }
 
     // po::notify(vm);
@@ -229,6 +239,7 @@ int main(int argc, char* argv[]) {
         {{"index", salmonIndex},
          {"quant", salmonQuantify},
          {"quantmerge", salmonQuantMerge},
+         // TODO : PF_INTEGRATION
          {"alevin", salmonBarcoding},
          {"swim", salmonSwim}});
 
@@ -280,7 +291,9 @@ int main(int argc, char* argv[]) {
         bool useSalmonAlign{false};
         for (int32_t i = 0; i < subCommandArgc; ++i) {
           if (strncmp(argv2[i], "-a", 2) == 0 or
-              strncmp(argv2[i], "--alignments", 12) == 0) {
+              strncmp(argv2[i], "-e", 2) == 0 or
+              strncmp(argv2[i], "--alignments", 12) == 0 or
+              strncmp(argv2[i], "--eqclasses", 11) == 0) {
             useSalmonAlign = true;
             break;
           }
