@@ -33,6 +33,7 @@ extern "C" {
 
 #include "AlignmentLibrary.hpp"
 #include "BAMQueue.hpp"
+#include "BAMUtils.hpp"
 #include "ClusterForest.hpp"
 #include "FASTAParser.hpp"
 #include "LibraryFormat.hpp"
@@ -60,6 +61,7 @@ extern "C" {
 #include "TextBootstrapWriter.hpp"
 #include "TranscriptCluster.hpp"
 #include "spdlog/spdlog.h"
+#include "pufferfish/Util.hpp"
 
 namespace bfs = boost::filesystem;
 using salmon::math::LOG_0;
@@ -67,6 +69,8 @@ using salmon::math::LOG_1;
 using salmon::math::logAdd;
 using salmon::math::logSub;
 
+using MateStatus = pufferfish::util::MateStatus;
+
 constexpr uint32_t miniBatchSize{1000};
 
 template <typename FragT> using AlignmentBatch = std::vector<FragT>;
@@ -112,6 +116,7 @@ inline bool tryToGetWork(MiniBatchQueue<AlignmentGroup<FragT*>>& workQueue,
   return foundWork;
 }
 
+
 template <typename FragT>
 void processMiniBatch(AlignmentLibraryT<FragT>& alnLib,
                       ForgettingMassCalculator& fmCalc,
@@ -184,7 +189,6 @@ void processMiniBatch(AlignmentLibraryT<FragT>& alnLib,
   auto& fragLengthDist = *(alnLib.fragmentLengthDistribution());
   auto& alnMod = alnLib.alignmentModel();
 
-  bool useFSPD{salmonOpts.useFSPD};
   bool useFragLengthDist{!salmonOpts.noFragLengthDist};
   bool noFragLenFactor{salmonOpts.noFragLenFactor};
 
@@ -197,6 +201,12 @@ void processMiniBatch(AlignmentLibraryT<FragT>& alnLib,
   uint32_t rangeFactorization{salmonOpts.rangeFactorizationBins};
 
   bool useAuxParams = (processedReads >= salmonOpts.numPreBurninFrags);
+  bool singleEndLib = !alnLib.isPairedEnd();
+  bool modelSingleFragProb = !salmonOpts.noSingleFragProb;
+  size_t prevProcessedReads{0};
+  size_t fragUpdateThresh{100000};
+
+  distribution_utils::LogCMFCache logCMFCache(&fragLengthDist, singleEndLib);
 
   std::chrono::microseconds sleepTime(1);
   MiniBatchInfo<AlignmentGroup<FragT*>>* miniBatch = nullptr;
@@ -210,6 +220,44 @@ void processMiniBatch(AlignmentLibraryT<FragT>& alnLib,
             !aln->isPaired());
   };
 
+  auto alignerType = alnLib.getAlignerType();
+
+  bool haveASTag{false};
+  bool firstTagCheck{true};
+  
+  // If we are dealing with RapMap or Pufferfish mappings, then we 
+  // don't expect CIGAR strings.  However, with recent versions 
+  // at least any version that we could consider to be reasonable 
+  // to run in alignment mode, we should have the `AS` tag. We set 
+  // this the variable so we know this and deal with 
+  // alignment score appropriately (and don't expect CIGAR strings).
+  // NOTE: If we are reading in pufferfish or RapMap alignments, we expect the AS tag,
+  // we will raise an exception if we don't have it.
+  bool useASWithoutCIGAR = ((alignerType == salmon::bam_utils::AlignerDetails::PUFFERFISH) or 
+                            (alignerType == salmon::bam_utils::AlignerDetails::RAPMAP));
+
+  auto getAlignerAssignedScore =
+    [&haveASTag, &firstTagCheck, alignerType](FragT* aln) -> double {
+      if (firstTagCheck and !haveASTag) {
+        char* tp = bam_aux_find(aln->getRead1(), "AS");
+        haveASTag = (tp != NULL);
+      }
+      double score{LOG_0};
+      if (haveASTag and (alignerType == salmon::bam_utils::AlignerDetails::BOWTIE2)) {
+        uint8_t* tl = reinterpret_cast<uint8_t*>(bam_aux_find(aln->getRead1(), "AS"));
+        auto locScore = (tl != NULL) ? bam_aux_i(tl) : LOG_1;
+        if (aln->isPaired()) {
+          uint8_t* tr = reinterpret_cast<uint8_t*>(bam_aux_find(aln->getRead2(), "AS"));
+          locScore += (tr != NULL) ? bam_aux_i(tr) : LOG_1;
+        }
+        score = locScore;
+      } else {
+        score = LOG_1;
+      }
+      firstTagCheck = false;
+      return score;
+    };
+
   while (!doneParsing or !workQueue.empty()) {
     uint32_t zeroProbFrags{0};
 
@@ -252,11 +300,20 @@ void processMiniBatch(AlignmentLibraryT<FragT>& alnLib,
       using HitIDVector = std::vector<size_t>;
       using HitProbVector = std::vector<double>;
 
+      // If we are going to attempt to model single mappings (part of a fragment)
+      // then cache the FLD cumulative distribution for this mini-batch.  If
+      // we are burned in or this is a single-end library, then the CMF is already
+      // cached, so we don't need to worry about doing this work ourselves.
+      if (modelSingleFragProb) {
+        logCMFCache.refresh(processedReads.load(), burnedIn.load());
+      }
+
       {
+        bool checkedASTag = false;
         // Iterate over each group of alignments (a group consists of all
         // alignments reported for a single read).  Distribute the read's mass
         // proportionally dependent on the current
-        for (auto alnGroup : alignmentGroups) {
+        for (auto& alnGroup : alignmentGroups) {
 
           // EQCLASS
           std::vector<uint32_t> txpIDs;
@@ -274,7 +331,35 @@ void processMiniBatch(AlignmentLibraryT<FragT>& alnLib,
               alnGroup->alignments().front()->transcriptID();
           std::unordered_set<size_t> observedTranscripts;
 
-          size_t sidx{0};
+          int32_t bestAS = std::numeric_limits<int32_t>::min();
+
+          // TODO: Use the AS tag to get the appropriate score for 
+          // reads aligned with pufferfish or RapMap.
+          
+          // If we are using the AS tag without CIGAR strings
+          if (useASWithoutCIGAR) {
+            // Loop over all of the alignments to find the maximum score, which 
+            // we will need to compute the alignment probability.
+            for (auto& aln : alnGroup->alignments()) {
+              if (!checkedASTag) {
+                checkedASTag = true;
+                if (! aln->haveASTag() ) {
+                  log->critical("salmon is being run in alignment mode with a SAM/BAM file generated by {}, but the "
+                  "alignment records do not seem to have the AS tag.  In this case, quantifying from alignments is not "
+                  "recommended or supported.", salmon::bam_utils::to_string(alignerType));
+                  spdlog::drop_all();
+                  std::exit(1);
+                }
+                haveASTag = true;
+              }
+
+              int32_t score = aln->getAS();
+              bestAS = (score > bestAS) ? score : bestAS;
+            }
+          }
+
+          //double maxLogAlnScore{LOG_0};
+          int sidx{0};
           for (auto& aln : alnGroup->alignments()) {
             auto transcriptID = aln->transcriptID();
             auto& transcript = refs[transcriptID];
@@ -292,9 +377,13 @@ void processMiniBatch(AlignmentLibraryT<FragT>& alnLib,
 
             // The probability of drawing a fragment of this length;
             double logFragProb = LOG_1;
-            // If we are expecting a paired-end library, and this is an orphan,
-            // then logFragProb should be small
-            if (isUnexpectedOrphan(aln, expectedLibraryFormat)) {
+            // if we are modeling fragment probabilities for single-end mappings
+            // and this is either a single-end library or an orphan.
+            if (modelSingleFragProb and useFragLengthDist and (singleEndLib or isUnexpectedOrphan(aln, expectedLibraryFormat))) {
+              logFragProb = logCMFCache.getAmbigFragLengthProb(aln->fwd(), aln->pos(), aln->readLen(), transcript.CompleteLength, burnedIn.load());
+            } else if (isUnexpectedOrphan(aln, expectedLibraryFormat)) {
+              // If we are expecting a paired-end library, and this is an orphan,
+              // then logFragProb should be small
               logFragProb = orphanProb;
               if (logFragProb == LOG_0) {
                 continue;
@@ -401,9 +490,14 @@ void processMiniBatch(AlignmentLibraryT<FragT>& alnLib,
             // Adjustment to the likelihood due to the
             // error model
             double errLike = salmon::math::LOG_1;
-            // if (burnedIn and salmonOpts.useErrorModel) {
-            if (useAuxParams and salmonOpts.useErrorModel) {
+            if (useASWithoutCIGAR) {
+              int32_t alnScore = aln->getAS();
+              // NOTE: we work directly in log space here, so the log
+              // prob is -scoreExp * (S-w) rather than exp^(-scoreExp * (S-w))
+              errLike = -salmonOpts.scoreExp * (bestAS - alnScore);
+            } else if (useAuxParams and salmonOpts.useErrorModel) {
               errLike = alnMod.logLikelihood(*aln, transcript);
+              ++sidx;
             }
 
             // Allow for a non-uniform fragment start position distribution
@@ -418,19 +512,6 @@ void processMiniBatch(AlignmentLibraryT<FragT>& alnLib,
             double fragStartLogDenominator{salmon::math::LOG_1};
 
             auto hitPos = aln->left();
-            if (useFSPD and burnedIn and hitPos < refLength) {
-              auto& fragStartDist =
-                  fragStartDists[transcript.lengthClassIndex()];
-              // Get the log(numerator) and log(denominator) for the fragment
-              // start position probability.
-              bool nonZeroProb = fragStartDist.logNumDenomMass(
-                  hitPos, refLength, logRefLength, fragStartLogNumerator,
-                  fragStartLogDenominator);
-              // Set the overall probability.
-              startPosProb = (nonZeroProb) ? fragStartLogNumerator -
-                                                 fragStartLogDenominator
-                                           : salmon::math::LOG_0;
-            }
 
             // The total auxiliary probabilty is the product (sum in log-space)
             // of The fragment length probabilty The mapping score (under error
@@ -509,7 +590,6 @@ void processMiniBatch(AlignmentLibraryT<FragT>& alnLib,
 
             double newMass = logForgettingMass + aln->logProb;
             transcript.addMass(newMass);
-            transcript.setLastTimestepUpdated(currentMinibatchTimestep);
 
             // ---- Collect seq-specific bias samples ------ //
             auto getCIGARLength = [](bam_seq_t* s) -> uint32_t {
@@ -652,7 +732,7 @@ void processMiniBatch(AlignmentLibraryT<FragT>& alnLib,
             if (posBiasCorrect) {
               auto lengthClassIndex = transcript.lengthClassIndex();
               switch (aln->mateStatus()) {
-              case rapmap::utils::MateStatus::PAIRED_END_PAIRED: {
+              case MateStatus::PAIRED_END_PAIRED: {
                 // TODO: Handle the non opposite strand case
                 if (aln->isInward()) {
                   auto* read1 = aln->getRead1();
@@ -673,9 +753,9 @@ void processMiniBatch(AlignmentLibraryT<FragT>& alnLib,
                                                               posRC, transcript.RefLength, aln->logProb);
                 }
               } break;
-              case rapmap::utils::MateStatus::PAIRED_END_LEFT:
-              case rapmap::utils::MateStatus::PAIRED_END_RIGHT:
-              case rapmap::utils::MateStatus::SINGLE_END: {
+              case MateStatus::PAIRED_END_LEFT:
+              case MateStatus::PAIRED_END_RIGHT:
+              case MateStatus::SINGLE_END: {
                 int32_t pos = aln->pos();
                 pos = pos < 0 ? 0 : pos;
                 pos = pos >= static_cast<int32_t>(transcript.RefLength) ?
@@ -777,8 +857,9 @@ void processMiniBatch(AlignmentLibraryT<FragT>& alnLib,
                           */
 
               // Update the error model
-              if (salmonOpts.useErrorModel) {
-                alnMod.update(*aln, transcript, LOG_1, logForgettingMass);
+              if (!useASWithoutCIGAR and salmonOpts.useErrorModel) {
+                auto alignerScore = getAlignerAssignedScore(aln);
+                alnMod.update(*aln, transcript, alignerScore, logForgettingMass);
               }
               // Update the fragment length distribution
               if (aln->isPaired() and !salmonOpts.noFragLengthDist) {
@@ -788,14 +869,6 @@ void processMiniBatch(AlignmentLibraryT<FragT>& alnLib,
                   fragLengthDist.addVal(fragLength, logForgettingMass);
                 }
               }
-              // Update the fragment start position distribution
-              if (useFSPD) {
-                auto hitPos = aln->left();
-                auto& fragStartDist =
-                    fragStartDists[transcript.lengthClassIndex()];
-                fragStartDist.addVal(hitPos, transcript.RefLength,
-                                     logForgettingMass);
-              }
             }
           }
 
@@ -878,13 +951,6 @@ void processMiniBatch(AlignmentLibraryT<FragT>& alnLib,
       --activeBatches;
       processedReads += batchReads;
       if (processedReads >= numBurninFrags and !burnedIn) {
-        if (useFSPD) {
-          // update all of the fragment start position
-          // distributions
-          for (auto& fspd : fragStartDists) {
-            fspd.update();
-          }
-        }
         // NOTE: only one thread should succeed here, and that
         // thread will set burnedIn to true
         alnLib.updateTranscriptLengthsAtomic(burnedIn);
@@ -1223,17 +1289,6 @@ bool quantifyLibrary(AlignmentLibraryT<FragT>& alnLib,
                               alnLib.numMappedFragments(),
                               salmonOpts.numBurninFrags);
 
-    // If we didn't have a sufficient number of samples for burnin,
-    // then also ignore modeling of the fragment start position
-    // distribution.
-    if (salmonOpts.useFSPD) {
-      salmonOpts.useFSPD = false;
-      salmonOpts.jointLog->warn("Since only {} (< {}) fragments were observed, "
-                                "modeling of the fragment start position "
-                                "distribution has been disabled",
-                                alnLib.numMappedFragments(),
-                                salmonOpts.numBurninFrags);
-    }
   }
 
   // In this case, we have to give the structures held
@@ -1265,6 +1320,10 @@ bool processSample(AlignmentLibraryT<ReadT>& alnLib, size_t requiredObservations
 
   bool burnedIn = false;
   try {
+    // if this is a single-end library, then the fld won't change
+    if (!alnLib.isPairedEnd()) {
+      alnLib.fragmentLengthDistribution()->cacheCMF();
+    }
     burnedIn = quantifyLibrary<ReadT>(alnLib, requiredObservations, sopt);
   } catch (const InsufficientAssignedFragments& iaf) {
     jointLog->warn(iaf.what());
@@ -1277,25 +1336,146 @@ bool processSample(AlignmentLibraryT<ReadT>& alnLib, size_t requiredObservations
     std::exit(1);
   }
 
-  // EQCLASS
-  // NOTE: A side-effect of calling the optimizer is that
-  // the `EffectiveLength` field of each transcript is
-  // set to its final value.
-  CollapsedEMOptimizer optimizer;
-  jointLog->info("starting optimizer");
-  salmon::utils::normalizeAlphas(sopt, alnLib);
-  bool optSuccess = optimizer.optimize(alnLib, sopt, 0.01, 10000);
-  // If the optimizer didn't work, then bail out here.
-  if (!optSuccess) {
-    return false;
-  }
-  jointLog->info("finished optimizer");
-
-  // EQCLASS
-  fmt::print(stderr, "\n\nwriting output \n");
   GZipWriter gzw(outputDirectory, jointLog);
-  // Write the main results
-  gzw.writeAbundances(sopt, alnLib);
+
+  if (!sopt.skipQuant) {
+    // NOTE: A side-effect of calling the optimizer is that
+    // the `EffectiveLength` field of each transcript is
+    // set to its final value.
+    CollapsedEMOptimizer optimizer;
+    jointLog->info("starting optimizer");
+    salmon::utils::normalizeAlphas(sopt, alnLib);
+    bool optSuccess = optimizer.optimize(alnLib, sopt, 0.01, 10000);
+    // If the optimizer didn't work, then bail out here.
+    if (!optSuccess) {
+      return false;
+    }
+    jointLog->info("finished optimizer");
+
+    jointLog->info("writing output");
+    // Write the main results
+    gzw.writeAbundances(sopt, alnLib, false);
+
+    if (sopt.numGibbsSamples > 0) {
+
+      jointLog->info("Starting Gibbs Sampler");
+      CollapsedGibbsSampler sampler;
+      gzw.setSamplingPath(sopt);
+      // The function we'll use as a callback to write samples
+      std::function<bool(const std::vector<double>&)> bsWriter =
+        [&gzw](const std::vector<double>& alphas) -> bool {
+          return gzw.writeBootstrap(alphas, true);
+        };
+
+      bool sampleSuccess =
+        sampler.sample(alnLib, sopt, bsWriter, sopt.numGibbsSamples);
+      if (!sampleSuccess) {
+        jointLog->error("Encountered error during Gibb sampling .\n"
+                        "This should not happen.\n"
+                        "Please file a bug report on GitHub.\n");
+        return false;
+      }
+      jointLog->info("Finished Gibbs Sampler");
+    } else if (sopt.numBootstraps > 0) {
+      // The function we'll use as a callback to write samples
+      std::function<bool(const std::vector<double>&)> bsWriter =
+        [&gzw](const std::vector<double>& alphas) -> bool {
+          return gzw.writeBootstrap(alphas);
+        };
+
+      jointLog->info("Staring Bootstrapping");
+      gzw.setSamplingPath(sopt);
+      bool bootstrapSuccess =
+        optimizer.gatherBootstraps(alnLib, sopt, bsWriter, 0.01, 10000);
+      jointLog->info("Finished Bootstrapping");
+      if (!bootstrapSuccess) {
+        jointLog->error("Encountered error during bootstrapping.\n"
+                        "This should not happen.\n"
+                        "Please file a bug report on GitHub.\n");
+        return false;
+      }
+    }
+
+    // bfs::path libCountFilePath = outputDirectory / "lib_format_counts.json";
+    // alnLib.summarizeLibraryTypeCounts(libCountFilePath);
+
+    if (sopt.sampleOutput) {
+      // In this case, we should "re-convert" transcript
+      // masses to be counts in log space
+      auto nr = alnLib.numMappedFragments();
+      for (auto& t : alnLib.transcripts()) {
+        double m = t.mass(false) * nr;
+        if (m > 0.0) {
+          t.setMass(std::log(m));
+        }
+      }
+
+      bfs::path sampleFilePath = outputDirectory / "postSample.bam";
+      bool didSample = salmon::sampler::sampleLibrary<ReadT>(
+                                                             alnLib, sopt, burnedIn, sampleFilePath, sopt.sampleUnaligned);
+      if (!didSample) {
+        jointLog->warn("There may have been a problem generating the sampled "
+                       "output file; please check the log\n");
+      }
+    }
+  } else if (sopt.dumpEqWeights) { // sopt.skipQuant == true
+    jointLog->info("Finalizing combined weights for equivalence classes.");
+      // if we are skipping the quantification, and we are dumping equivalence class weights,
+      // then fill in the combinedWeights of the equivalence classes so that `--dumpEqWeights` makes sense.
+      auto& eqVec =
+        alnLib.equivalenceClassBuilder().eqVec();
+      bool noRichEq = sopt.noRichEqClasses;
+      bool useEffectiveLengths = !sopt.noEffectiveLengthCorrection;
+      std::vector<Transcript>& transcripts = alnLib.transcripts();
+      Eigen::VectorXd effLens(transcripts.size());
+
+      for (size_t i = 0; i < transcripts.size(); ++i) {
+        auto& txp = transcripts[i];
+        effLens(i) = useEffectiveLengths
+          ? std::exp(txp.getCachedLogEffectiveLength())
+          : txp.RefLength;
+      }
+
+      for (size_t eqID = 0; eqID < eqVec.size(); ++eqID){
+        // The vector entry
+        auto& kv = eqVec[eqID];
+        // The label of the equivalence class
+        const TranscriptGroup& k = kv.first;
+        // The size of the label
+        size_t classSize = kv.second.weights.size(); // k.txps.size();
+        // The weights of the label
+        auto& v = kv.second;
+
+        // Iterate over each weight and set it
+        double wsum{0.0};
+
+        for (size_t i = 0; i < classSize; ++i) {
+          auto tid = k.txps[i];
+          double el = effLens(tid);
+          if (el <= 1.0) {
+            el = 1.0;
+          }
+          if (noRichEq) {
+            // Keep length factor separate for the time being
+            v.weights[i] = 1.0;
+          }
+          // meaningful values.
+          auto probStartPos = 1.0 / el;
+
+          // combined weight
+          double wt = sopt.eqClassMode ? v.weights[i] : v.count * v.weights[i] * probStartPos;
+          v.combinedWeights.push_back(wt);
+          wsum += wt;
+        }
+
+        double wnorm = 1.0 / wsum;
+        for (size_t i = 0; i < classSize; ++i) {
+          v.combinedWeights[i] = v.combinedWeights[i] * wnorm;
+        }
+      }
+      jointLog->info("done.");
+    }
+
 
   // If we are dumping the equivalence classes, then
   // do it here.
@@ -1303,72 +1483,45 @@ bool processSample(AlignmentLibraryT<ReadT>& alnLib, size_t requiredObservations
     gzw.writeEquivCounts(sopt, alnLib);
   }
 
-  if (sopt.numGibbsSamples > 0) {
-
-    jointLog->info("Starting Gibbs Sampler");
-    CollapsedGibbsSampler sampler;
-    gzw.setSamplingPath(sopt);
-    // The function we'll use as a callback to write samples
-    std::function<bool(const std::vector<double>&)> bsWriter =
-        [&gzw](const std::vector<double>& alphas) -> bool {
-      return gzw.writeBootstrap(alphas);
-    };
-
-    bool sampleSuccess =
-        sampler.sample(alnLib, sopt, bsWriter, sopt.numGibbsSamples);
-    if (!sampleSuccess) {
-      jointLog->error("Encountered error during Gibb sampling .\n"
-                      "This should not happen.\n"
-                      "Please file a bug report on GitHub.\n");
-      return false;
-    }
-    jointLog->info("Finished Gibbs Sampler");
-  } else if (sopt.numBootstraps > 0) {
-    // The function we'll use as a callback to write samples
-    std::function<bool(const std::vector<double>&)> bsWriter =
-        [&gzw](const std::vector<double>& alphas) -> bool {
-      return gzw.writeBootstrap(alphas);
-    };
-
-    jointLog->info("Staring Bootstrapping");
-    gzw.setSamplingPath(sopt);
-    bool bootstrapSuccess =
-        optimizer.gatherBootstraps(alnLib, sopt, bsWriter, 0.01, 10000);
-    jointLog->info("Finished Bootstrapping");
-    if (!bootstrapSuccess) {
-      jointLog->error("Encountered error during bootstrapping.\n"
-                      "This should not happen.\n"
-                      "Please file a bug report on GitHub.\n");
-      return false;
-    }
-  }
-
-  // bfs::path libCountFilePath = outputDirectory / "lib_format_counts.json";
-  // alnLib.summarizeLibraryTypeCounts(libCountFilePath);
-
-  if (sopt.sampleOutput) {
-    // In this case, we should "re-convert" transcript
-    // masses to be counts in log space
-    auto nr = alnLib.numMappedFragments();
-    for (auto& t : alnLib.transcripts()) {
-      double m = t.mass(false) * nr;
-      if (m > 0.0) {
-        t.setMass(std::log(m));
-      }
-    }
-
-    bfs::path sampleFilePath = outputDirectory / "postSample.bam";
-    bool didSample = salmon::sampler::sampleLibrary<ReadT>(
-        alnLib, sopt, burnedIn, sampleFilePath, sopt.sampleUnaligned);
-    if (!didSample) {
-      jointLog->warn("There may have been a problem generating the sampled "
-                     "output file; please check the log\n");
-    }
-  }
-
   sopt.runStopTime = salmon::utils::getCurrentTimeAsString();
   // Write meta-information about the run
-  gzw.writeMeta(sopt, alnLib);
+  MappingStatistics mstats;
+  gzw.writeMeta(sopt, alnLib, mstats);
+
+  return true;
+}
+
+bool processEqClasses( AlignmentLibraryT<UnpairedRead>& alnLib, SalmonOpts& sopt,
+                       boost::filesystem::path outputDirectory) {
+  auto& jointLog = sopt.jointLog;
+  GZipWriter gzw(outputDirectory, jointLog);
+
+  // NOTE: A side-effect of calling the optimizer is that
+  // the `EffectiveLength` field of each transcript is
+  // set to its final value.
+  CollapsedEMOptimizer optimizer;
+  jointLog->info("starting optimizer");
+
+  {
+    // setting no effective length correction, as we are taking effective lens as input
+    sopt.initUniform = true;
+    sopt.eqClassMode = true;
+    jointLog->warn("Using Uniform Prior");
+  }
+
+  //salmon::utils::normalizeAlphas(sopt, alnLib);
+  bool optSuccess = optimizer.optimize(alnLib, sopt, 0.01, 10000);
+  // If the optimizer didn't work, then bail out here.
+  if (!optSuccess) {
+    return false;
+  }
+  jointLog->info("finished optimizer");
+  jointLog->info("writing output");
+  jointLog->flush();
+
+  // Write the main results
+  gzw.writeAbundances(sopt, alnLib, true);
+  sopt.runStopTime = salmon::utils::getCurrentTimeAsString();
 
   return true;
 }
@@ -1429,6 +1582,25 @@ transcript abundance from RNA-seq reads
     }
     auto numThreads = sopt.numThreads;
 
+    bool hasAlignments {false};
+    bool hasEqclasses {false};
+    vector<string> alignmentFileNames;
+    string eqclassesFileName;
+    if (vm.count("alignments")) {
+      alignmentFileNames = vm["alignments"].as<vector<string>>();
+      hasAlignments = true;
+    }
+    if (vm.count("eqclasses")) {
+      eqclassesFileName = vm["eqclasses"].as<string>();
+      hasEqclasses = true;
+    }
+
+    if ( !hasAlignments and !hasEqclasses ) {
+      fmt::print(stderr, "salmon requires at least one alignment input\n"
+                 "Neither alignments (BAM) nor eqclasses given as input. \n");
+      std::exit(1);
+    }
+
     if (sopt.forgettingFactor <= 0.5 or sopt.forgettingFactor > 1.0) {
       fmt::print(stderr,
                  "The forgetting factor must be in (0.5, 1.0], "
@@ -1456,7 +1628,7 @@ transcript abundance from RNA-seq reads
     sopt.alnMode = true;
     sopt.quantMode = SalmonQuantMode::ALIGN;
     bool optionsOK =
-        salmon::utils::processQuantOptions(sopt, vm, numBiasSamples);
+      salmon::utils::processQuantOptions(sopt, vm, numBiasSamples);
     if (!optionsOK) {
       if (sopt.jointLog) {
         sopt.jointLog->flush();
@@ -1471,13 +1643,25 @@ transcript abundance from RNA-seq reads
     auto outputDirectory = sopt.outputDirectory;
 
     // ==== Library format processing ===
-    vector<string> alignmentFileNames = vm["alignments"].as<vector<string>>();
     vector<bfs::path> alignmentFiles;
-    for (auto& alignmentFileName : alignmentFileNames) {
-      bfs::path alignmentFile(alignmentFileName);
+
+    if ( hasAlignments ) {
+      for (auto& alignmentFileName : alignmentFileNames) {
+        bfs::path alignmentFile(alignmentFileName);
+        if (!bfs::exists(alignmentFile)) {
+          std::stringstream ss;
+          ss << "The provided alignment file: " << alignmentFile
+             << " does not exist!\n";
+          throw std::invalid_argument(ss.str());
+        } else {
+          alignmentFiles.push_back(alignmentFile);
+        }
+      }
+    } else {
+      bfs::path alignmentFile(eqclassesFileName);
       if (!bfs::exists(alignmentFile)) {
         std::stringstream ss;
-        ss << "The provided alignment file: " << alignmentFile
+        ss << "The provided eqclasses file: " << alignmentFile
            << " does not exist!\n";
         throw std::invalid_argument(ss.str());
       } else {
@@ -1490,23 +1674,29 @@ transcript abundance from RNA-seq reads
                          ReadStrandedness::U);
     // Get the library format string
     std::string libFmtStr = vm["libType"].as<std::string>();
-
-    // If we're auto-detecting, set things up appropriately
     bool autoDetectFmt =
-        (libFmtStr == "a" or
-         libFmtStr == "A"); //(autoTypes.find(libFmtStr) != autoTypes.end());
-    if (autoDetectFmt) {
+      (libFmtStr == "a" or
+       libFmtStr == "A"); //(autoTypes.find(libFmtStr) != autoTypes.end());
 
-      bool isPairedEnd = salmon::utils::peekBAMIsPaired(alignmentFiles.front());
-      if (isPairedEnd) {
-        libFmt = LibraryFormat(ReadType::PAIRED_END, ReadOrientation::TOWARD,
-                               ReadStrandedness::U);
-      } else {
-        libFmt = LibraryFormat(ReadType::SINGLE_END, ReadOrientation::NONE,
-                               ReadStrandedness::U);
+    if ( hasEqclasses ) {
+      libFmt = LibraryFormat(ReadType::SINGLE_END, ReadOrientation::NONE,
+                             ReadStrandedness::U);
+    } else {
+      // If we're auto-detecting, set things up appropriately
+      if (autoDetectFmt) {
+
+        bool isPairedEnd = salmon::utils::peekBAMIsPaired(alignmentFiles.front());
+
+        if (isPairedEnd) {
+          libFmt = LibraryFormat(ReadType::PAIRED_END, ReadOrientation::TOWARD,
+                                 ReadStrandedness::U);
+        } else {
+          libFmt = LibraryFormat(ReadType::SINGLE_END, ReadOrientation::NONE,
+                                 ReadStrandedness::U);
+        }
+      } else { // Parse the provided type
+        libFmt = salmon::utils::parseLibraryFormatStringNew(libFmtStr);
       }
-    } else { // Parse the provided type
-      libFmt = salmon::utils::parseLibraryFormatStringNew(libFmtStr);
     }
 
     if (libFmt.check()) {
@@ -1519,7 +1709,8 @@ transcript abundance from RNA-seq reads
     // ==== END: Library format processing ===
 
     // The transcript file contains the target sequences
-    bfs::path transcriptFile(vm["targets"].as<std::string>());
+    bfs::path transcriptFile;
+    if (hasAlignments) { transcriptFile = vm["targets"].as<std::string>(); }
 
     // Currently, one thread is used for parsing the alignment file.
     // Hopefully, in the future, samtools will implemented multi-threaded
@@ -1553,16 +1744,80 @@ transcript abundance from RNA-seq reads
         // sopt.gcBiasCorrect = false;
       }
 
-      AlignmentLibraryT<UnpairedRead> alnLib(alignmentFiles, transcriptFile,
-                                            libFmt, sopt);
+      if ( hasEqclasses ) {
+        std::vector<string> tnames;
+        std::vector<double> tefflens;
+        std::vector<uint32_t> eqclass_counts;
+        std::vector<std::vector<uint32_t>> eqclasses;
+        std::vector<std::vector<double>> auxs_vals;
+        {
+          // reading eqclass
+          bool parseOK = salmon::utils::readEquivCounts(alignmentFiles[0], tnames, tefflens,
+                                                        eqclasses, auxs_vals, eqclass_counts);
+          if (!parseOK){
+            jointLog->error("Eqclass Parsing error");
+            exit(1);
+          }
 
-      if (autoDetectFmt) {
-        alnLib.enableAutodetect();
+          std::stringstream errfmt;
+          errfmt << "Found total " << eqclasses.size() << " eqclasses and "
+                 << tnames.size() << " transcripts";
+          jointLog->info(errfmt.str());
+          jointLog->flush();
+
+          if ( tefflens.size() == 0 ) {
+            tefflens.resize(tnames.size(), 100);
+            jointLog->warn("No effective lens found in the eqclass file;"
+                           "Ignore this warning if using uniform prior");
+          }
+
+          if ( /*tefflens.size() != 0 and*/ (tefflens.size() != tnames.size()) ){
+            std::stringstream errfmt;
+            errfmt << "Number of effective lens: " << tefflens.size()
+                   << " is not equal to number of transcripts: " << tnames.size();
+            jointLog->error(errfmt.str());
+            jointLog->flush();
+            exit(1);
+          }
+
+          if ( eqclasses.size() != auxs_vals.size() or
+               eqclasses.size() != eqclass_counts.size() ) {
+            jointLog->error("Different size of the eqclasses object");
+            jointLog->flush();
+            exit(1);
+          }
+        }
+
+        AlignmentLibraryT<UnpairedRead> alnLib(alignmentFiles,
+                                               libFmt, sopt, hasEqclasses,
+                                               tnames, tefflens);
+
+        jointLog->info("Created AlignmentLibrary object");
+        jointLog->flush();
+
+        // EQCLASS
+        alnLib.equivalenceClassBuilder().populateTargets(eqclasses, auxs_vals,
+                                                         eqclass_counts,
+                                                         alnLib.transcripts());
+        success = processEqClasses(alnLib, sopt, outputDirectory);
+      } else {
+        AlignmentLibraryT<UnpairedRead> alnLib(alignmentFiles, transcriptFile,
+                                               libFmt, sopt);
+
+        if (autoDetectFmt) {
+          alnLib.enableAutodetect();
+        }
+        success = processSample<UnpairedRead>(alnLib, requiredObservations, sopt,
+                                              outputDirectory);
       }
-      success = processSample<UnpairedRead>(alnLib, requiredObservations, sopt,
-                                            outputDirectory);
     } break;
     case ReadType::PAIRED_END: {
+      if (hasEqclasses) {
+        jointLog->error(" Cannot quantify eqclasses in mode \n"
+                        " Please report this on github");
+        std::exit(1);
+      }
+
       AlignmentLibraryT<ReadPair> alnLib(alignmentFiles, transcriptFile, libFmt,
                                         sopt);
       if (autoDetectFmt) {
